/****************************************************************
 *                                                              *
 *  Copyright (C) 2003-2009 Christoffer Owe                     *
 *  For conditions of distribution and use, see License.txt     *
 *                                                              *
 ****************************************************************/

// GED2XML.cpp : Defines the entry point for the console application.
//

#include <windows.h>
#include <stdio.h>
#include <Shlwapi.h>
#include <time.h>
#include "ged2xml.h"

// Error messages
static TCHAR* ErrorMessage[17]={
	TEXT(""),
	TEXT("Unexpected end of file. TRLR record was missing."),
	TEXT("User-defined tags are ignored."),
	TEXT("The tag is unknown."),
	TEXT("The header record is missing."),
	TEXT("Unexpected tag."),
	TEXT("The line does not conform to the GEDCOM standard."),
	TEXT("The GEDCOM file was not found."),
	TEXT("The XREF is missing."),
	TEXT("The XREF is too long."),
	TEXT("Date format is invalid."),
	TEXT("The conversion was aborted."),
	TEXT("Unknown character set. ANSEL assumed."),
	TEXT("Warning: BETWEEN detected in date. Translated to BET."),
	TEXT("Warning: CA, CA. or CIR detected in date. Translated to ABT."),
	TEXT("Warning: TIME detected after date. Imported as part of date, but not standard GEDCOM."),
	TEXT("A pointer was expected after this tag.")};
static TCHAR IgnoreMessage[]=TEXT("The line was ignored.");
// Default event type descriptions
static TCHAR* EventType[51]={
	TEXT("born"),
	TEXT("baptised"),
	TEXT("died"),
	TEXT("buried"),
	TEXT("cremated"),
	TEXT("adopted"),
	TEXT("baptised"),
	TEXT("bar mitzvah"),
	TEXT("bas mitzvah"),
	TEXT("blessed"),
	TEXT("adult christening"),
	TEXT("confirmated"),
	TEXT("first communion"),
	TEXT("ordinated"),
	TEXT("naturalized"),
	TEXT("emigrated"),
	TEXT("immigrated"),
	TEXT("census"),
	TEXT("probated"),
	TEXT("will signed"),
	TEXT("graduated"),
	TEXT("retired"),
	TEXT("residence"),
	TEXT("caste"),
	TEXT("description"),
	TEXT("education"),
	TEXT("ID number"),
	TEXT("nationality"),
	TEXT("number of children"),
	TEXT("number of marriages"),
	TEXT("occupation"),
	TEXT("property"),
	TEXT("religion"),
	TEXT("social security number"),
	TEXT("title"),
	TEXT("marriage annuled"),
	TEXT("divorced"),
	TEXT("filed for divorce"),
	TEXT("engaged"),
	TEXT("married"),
	TEXT("marriage bann"),
	TEXT("marriage contract"),
	TEXT("marriage license"),
	TEXT("marriage settlement"),
	TEXT("baptism (LDS)"),
	TEXT("confirmation (LDS)"),
	TEXT("endowment (LDS)"),
	TEXT("child sealing (LDS)"),
	TEXT("spouse sealing (LDS)"),
	TEXT("email"),
	TEXT("UID")};

void about()
{
	wprintf(TEXT("Converts GEDCOM-files to GenXML 2.0.\r\n\r\n"));
	wprintf(TEXT("Usage: GED2XML [options] Gedcomfile GenXMLfile\r\n\r\n"));
	wprintf(TEXT("         Gedcomfile     GEDCOM-file to convert\r\n"));
	wprintf(TEXT("         GenXMLfile     GenXML-file to create (must not exist)\r\n\r\n"));
	wprintf(TEXT("    Options:\r\n"));
	wprintf(TEXT("         -0             Report all messages (default)\r\n"));
	wprintf(TEXT("         -1             Report less\r\n"));
	wprintf(TEXT("         -2             Report errors only\r\n"));
	wprintf(TEXT("         -a             Treat ALIA as NAME\r\n"));
	wprintf(TEXT("         -c             Assume Gregorian calendar unless specified\r\n"));
	wprintf(TEXT("         -d             Treat erroneous dates as text\r\n"));
	wprintf(TEXT("         -e0            Ignore and log event info as AGE, AGNC and CAUS\r\n"));
	wprintf(TEXT("         -e1            Ignore and don't log event info as AGE, AGNC and CAUS\r\n"));
	wprintf(TEXT("         -e2            Treat event info as AGE, AGNC and CAUS as notes\r\n"));
	wprintf(TEXT("         -g             Assume name before surname is given name\r\n"));
	wprintf(TEXT("         -m0            Embedded objects\r\n"));
	wprintf(TEXT("         -m1            No embedded objects\r\n"));
	wprintf(TEXT("         -m2            Keep current embedded status (default)\r\n"));
	wprintf(TEXT("         -n             Use structured names if present\r\n"));
	wprintf(TEXT("         -p0            Ignore and log person info as RESN, AFN and RIN\r\n"));
	wprintf(TEXT("         -p1            Ignore and don't log person info as RESN, AFN and RIN\r\n"));
	wprintf(TEXT("         -p2            Treat person info as RESN, AFN and RIN as notes\r\n"));
	wprintf(TEXT("         -s0            Character set for output: ANSI (default)\r\n"));
	wprintf(TEXT("         -s1            Character set for output: UTF-8\r\n"));
	wprintf(TEXT("         -s2            Character set for output: UTF-16\r\n"));
	wprintf(TEXT("         -t             Print time used for conversion\r\n"));
	wprintf(TEXT("         -u             Try handling user defined tags\r\n"));
	wprintf(TEXT("\r\n"));

}

TCHAR* Messages(int msg)
{
	if (msg<ERROR_IGNORE)
		return ErrorMessage[msg];
	if (msg==ERROR_IGNORE)
		return IgnoreMessage;
	return EventType[msg-1000];
}

int wmain(int argc, wchar_t* argv[])
{
	static TCHAR szSuccess[]=TEXT("\r\nGEDCOM-file successfully converted to GenXML!\r\n\r\n");
	static TCHAR szGedcomFailure[]=TEXT("\r\nErrors detected parsing GEDCOM-file or file not found.\r\n\r\n");
	static TCHAR szGenxmlFailure[]=TEXT("\r\nErrors detected writing GenXML-file.\r\n\r\n");
	static TCHAR szTime[]=TEXT("The conversion was done in %i minutes and %i seconds\r\n\r\n");
	CONVERSIONDATA Conv;
	int i;
	BOOL fTime = FALSE;
	clock_t time;

	wprintf(TEXT("Ged2XML version 2.0\r\n"));
	wprintf(TEXT("Copyright (C) Christoffer Owe 2003-2009\r\n\r\n"));
	// Display help if too few arguments
	if (argc<3)
	{
		about();
		return 0;
	}

	// Convert the file
	//Conv.hLogFile = CreateFile("CON",GENERIC_WRITE,0,NULL,OPEN_EXISTING,0,NULL);
	Conv.StructSize = sizeof(Conv);
	Conv.hLogFile = GetStdHandle(STD_OUTPUT_HANDLE);
	Conv.LogFileIsConsole = TRUE;
	Conv.MsgProvider = Messages;
	Conv.MessageLevel = MESSAGELEVEL_ALL;
	Conv.AssumeGivenName = FALSE;
	Conv.UseStructuredNames = FALSE;
	Conv.AssumeGregorian = FALSE;
	Conv.TextDates = FALSE;
	Conv.AliasNames = FALSE;
	Conv.UserDefinedTags = FALSE;
	Conv.ObjectStatus = 2;
	Conv.EventInfoStatus = 0;
	Conv.PersonInfoStatus = 0;
	Conv.CharacterSet = 0;
	i = 1;
	while (argv[i][0]=='-')
	{
		if (!StrCmp(argv[i],TEXT("-t")))
			fTime = TRUE;
		else if (!StrCmp(argv[i],TEXT("-a")))
			Conv.AliasNames = TRUE;
		else if (!StrCmp(argv[i],TEXT("-g")))
			Conv.AssumeGivenName = TRUE;
		else if (!StrCmp(argv[i],TEXT("-n")))
			Conv.UseStructuredNames = TRUE;
		else if (!StrCmp(argv[i],TEXT("-c")))
			Conv.AssumeGregorian = TRUE;
		else if (!StrCmp(argv[i],TEXT("-d")))
			Conv.TextDates = TRUE;
		else if (!StrCmp(argv[i],TEXT("-u")))
			Conv.UserDefinedTags = TRUE;
		else if (!StrCmp(argv[i],TEXT("-m0")))
			Conv.ObjectStatus = 0;
		else if (!StrCmp(argv[i],TEXT("-m1")))
			Conv.ObjectStatus = 1;
		else if (!StrCmp(argv[i],TEXT("-e0")))
			Conv.EventInfoStatus = 0;
		else if (!StrCmp(argv[i],TEXT("-e1")))
			Conv.EventInfoStatus = 1;
		else if (!StrCmp(argv[i],TEXT("-e2")))
			Conv.EventInfoStatus = 2;
		else if (!StrCmp(argv[i],TEXT("-p0")))
			Conv.PersonInfoStatus = 0;
		else if (!StrCmp(argv[i],TEXT("-p1")))
			Conv.PersonInfoStatus = 1;
		else if (!StrCmp(argv[i],TEXT("-p2")))
			Conv.PersonInfoStatus = 2;
		else if (!StrCmp(argv[i],TEXT("-s0")))
			Conv.CharacterSet = 0;
		else if (!StrCmp(argv[i],TEXT("-s1")))
			Conv.CharacterSet = 1;
		else if (!StrCmp(argv[i],TEXT("-s2")))
			Conv.CharacterSet = 2;
		else if (iswdigit(argv[i][1]))
		{
			Conv.MessageLevel = -_wtoi(argv[i]);
			if (Conv.MessageLevel<MESSAGELEVEL_ALL)
				Conv.MessageLevel=MESSAGELEVEL_ALL;
			if (Conv.MessageLevel>MESSAGELEVEL_SKIPERRORS)
				Conv.MessageLevel=MESSAGELEVEL_SKIPERRORS;
		}
		i++;
	}
	Conv.szGedcomFile = argv[i];
	Conv.szGenXMLFile = argv[i+1];
	int result = Convert(&Conv);
	//CloseHandle(Conv.hLogFile);
	if (result==RETURN_OK)
		wprintf(szSuccess);
	else if (result==RETURN_GEDCOMFAILURE)
		wprintf(szGedcomFailure);
	else if (result==RETURN_GENXMLFAILURE)
		wprintf(szGenxmlFailure);
	if (fTime)
	{
		time = clock()/CLOCKS_PER_SEC;
		wprintf(szTime,time/60,time%60);
	}
	return 0;
}
