/****************************************************************
 *                                                              *
 *  Copyright (C) 2003-2007 Christoffer Owe                     *
 *  For conditions of distribution and use, see License.txt     *
 *                                                              *
 ****************************************************************/

#include <Windows.h>
#include <StrSafe.h>
#include "ged2xml.h"
#include "gc_event.h"

int EVENTTYPE::Find(VECTOR<EVENTTYPE*>& list,int tag,TCHAR* descr)
{
	for (long i=1;i<=list.GetSize();i++)
	{
		if (list.Get(i)->tag==tag && (!descr || list.Get(i)->description==descr))
			return (i);
	}
	return 0;
}

BOOL GC_EVENT::Import(GEDCOMFILE& file,VECTOR<EVENTTYPE*>& Eventtypes,VECTOR<GC_SOURCE*>& Sources,VECTOR<GC_SOURCECITE*>& Sourcerefs,int iRoles)
{
	TCHAR szType[256];
	EVENTTYPE* et;
	GC_SOURCECITE* Sourceref;
	BOOL result = TRUE;

	int TopLevel = file.GetLevel();
	szType[0] = 0;
	Tag = file.GetTag();
	file.NextLine();
	if (!file.InterpretLine())
		return (FALSE);
	while (file.GetLevel()>TopLevel && result==TRUE)
	{
		switch(file.GetTag())
		{
		case TAG_AGE:
		case TAG_AGNC:
		case TAG_CAUS:
			if (file.ConvData->EventInfoStatus==0)
			{
				file.PrintError(ERROR_IGNORE,TRUE);
				//result = file.NextLine();
			}
			//else if (file.ConvData->EventInfoStatus==1)
			//	result = file.NextLine();
			else if (file.ConvData->EventInfoStatus==2)
			{
				if (Note.Note.Size())
					Note.Note.Add(TEXT("\r\n"));
				if (file.GetTag()==TAG_AGE)
					Note.Note.Add(TEXT("Age "));
				else if (file.GetTag()==TAG_AGNC)
					Note.Note.Add(TEXT("Agency "));
				Note.Note.Add(file.GetText());
				Note.Note.Add(TEXT("."));
			}
			result = file.NextLine();
			break;
		case TAG_TYPE:
			StringCbCopy(szType,sizeof(szType),file.GetText());
			szType[sizeof(szType)-1] = 0;
			result = file.NextLine();
			break;
		case TAG_STAT:		//LDS events only
			Note.Note.Add(file.GetText());
			Note.Note.Add(TEXT(". "));
			result = file.NextLine();
			break;
		case TAG_TEMP:		//LDS events only
			Note.Note.Add(file.GetText());
			Note.Note.Add(TEXT(". "));
			result = file.NextLine();
			break;
		case TAG_DATE:
			result = Date.Import(file);
			break;
		case TAG_PLAC:
			result = Place.Import(file);
			break;
		case TAG_ADDR:
			result = Address.Import(file);
			break;
		case TAG_NOTE:
			result = Note.ImportNote(file);
			break;
		case TAG_SOUR:
			Sourceref = new GC_SOURCECITE();
			result = Sourceref->Import(file,Sources);
			SourcerefIDs.Add(Sourcerefs.Add(Sourceref));
			break;
		case TAG_QUAY:	// Known ROOTSIII error
			if (file.GetMsgLevel()<MESSAGELEVEL_SKIPERRORS)
				result = file.IllegalTag();
			else
				result = file.NextLine();
			break;
		case TAG_FAMC:
			if (Tag==TAG_ADOP)
			{
				file.PrintError(ERROR_IGNORE,TRUE);
				result = file.SkipStructure();
			}
			else
				result = file.IllegalTag();
			break;
		default:
			result = file.IllegalTag();
			break;
		}
		if (!result)
			return FALSE;
		if (!file.InterpretLine())
			return FALSE;
	}
	// Add event type
	EventType = EVENTTYPE::Find(Eventtypes,Tag,szType);
	if (EventType)
		return TRUE;	// Event type already added
	et = new EVENTTYPE;
	et->tag = Tag;
	et->roles = iRoles;
	et->description = szType;
	EventType = Eventtypes.Add(et);
	return TRUE;
}

// Import address and phone at level 1 and store as RESI event (Brother's Keepers)
BOOL GC_EVENT::ImportAddress(GEDCOMFILE& file,VECTOR<EVENTTYPE*>& Eventtypes)
{
	EVENTTYPE* et;
	BOOL result = TRUE;

	result = Address.Import(file);
	if (!result)
		return FALSE;
	if (!file.InterpretLine())
		return FALSE;
	// Add event type
	EventType = EVENTTYPE::Find(Eventtypes,TAG_RESI,NULL);
	if (EventType)
		return TRUE;	// Event type already added
	et = new EVENTTYPE;
	et->tag = TAG_RESI;
	et->roles = 0;
	//et->description = szType;
	EventType = Eventtypes.Add(et);
	return TRUE;
}