/****************************************************************
 *                                                              *
 *  Copyright (C) 2003-2009 Christoffer Owe                     *
 *  For conditions of distribution and use, see License.txt     *
 *                                                              *
 ****************************************************************/

#include <Windows.h>
#include <Shlwapi.h>
#include "gc_header.h"
#include "gedcom.h"
#include "ged2xml.h"

BOOL GC_HEADER::Import(GEDCOMFILE& file)
{
	int Level1Tag=0;
	int Level2Tag=0;

	file.NextLine();
	if (!file.InterpretLine())
		return (FALSE);
	while (file.GetLevel()>0)
	{
		if (file.GetLevel()==1)
			Level1Tag = file.GetTag();
		else if (file.GetLevel()==2)
			Level2Tag = file.GetTag();
		switch(file.GetTag())
		{
		case TAG_SOUR:
			Sour = file.GetText();
			if (Sour==TEXT("Legacy") || Sour==TEXT("LEGACY"))
				file.SetSystem(SYSTEM_LEGACY);
			else if (Sour==TEXT("FamilyOrigins"))
				file.SetSystem(SYSTEM_FAMILYORIGINS);
			else if (Sour==TEXT("BROSKEEP"))
				file.SetSystem(SYSTEM_BROTHERSKEEPERS);
			if (!file.NextLine())
				return FALSE;
			break;
		case TAG_VERS:
			if (Level1Tag==TAG_SOUR)
				SourVers = file.GetText();
			else if (Level1Tag==TAG_GEDC)
				GedcVers = file.GetText();
			else if (Level1Tag==TAG_CHAR)
				CharVers = file.GetText();
			if (!file.NextLine())
				return FALSE;
			break;
		case TAG_NAME:
			SourName = file.GetText();
			if (!file.NextLine())
				return FALSE;
			break;
		case TAG_CORP:
			SourCorp = file.GetText();
			if (!file.NextLine())
				return FALSE;
			if (!SourCorpAddress.Import(file))
				return FALSE;
			break;
		case TAG_DEST:
			Dest = file.GetText();
			if (!file.NextLine())
				return FALSE;
			break;
		case TAG_DATE:
			if (!Date.Import(file))
				return FALSE;
			// Unknown characters?
			/*while (file.GetText()[0]==' ')
				file.AddIndex(1);
			if (file.GetText()[0])
				file.PrintError(ERROR_DATEFORMAT,TRUE);*/
			break;
		case TAG_SUBM:
			StrCpy(Submitter,file.GetPointer());
			if (!file.NextLine())
				return FALSE;
			break;
		case TAG_FILE:
			File = file.GetText();
			if (!file.NextLine())
				return FALSE;
			break;
		case TAG_GEDC:
			if (!file.NextLine())
				return FALSE;
			break;
		case TAG_FORM:
			GedcForm = file.GetText();
			if (!file.NextLine())
				return FALSE;
			break;
		case TAG_CHAR:
			Char = file.GetText();
			if (!file.NextLine())
				return FALSE;
			break;
		case TAG_COPR:
			Copr = file.GetText();
			if (!file.NextLine())
				return FALSE;
			break;
		case TAG_DATA:
		case TAG_PLAC:
		case TAG_NOTE:
			if (!file.SkipStructure())
				return FALSE;
		default:
			if (!file.IllegalTag())
				return FALSE;
			break;
		}
		if (!file.InterpretLine())
			return FALSE;
	}
	return TRUE;
}