/****************************************************************
 *                                                              *
 *  Copyright (C) 2003-2009 Christoffer Owe                     *
 *  For conditions of distribution and use, see License.txt     *
 *                                                              *
 ****************************************************************/

#include <Windows.h>
#include <Shlwapi.h>
#include "gc_note.h"
#include "gc_source.h"

BOOL GC_NOTEREC::Import(GEDCOMFILE& file,IDTABLE* NoteIDs)
{
	BOOL result = TRUE;

	NoteIDs->Add(file.GetXRef());
	Note.Add(file.GetText());
	if (!file.NextLine())
		return FALSE;
	if (!file.InterpretLine())
		return FALSE;
	while (file.GetLevel()>0)
	{
		switch(file.GetTag())
		{
		case TAG_CONT:
			Note.Add(TEXT("\r\n"));
			Note.Add(file.GetText());
			result = file.NextLine();
			break;
		case TAG_CONC:
			Note.Add(file.GetText());
			result = file.NextLine();
			break;
		case TAG_SOUR:
		case TAG_REFN:
		case TAG_CHAN:
			file.PrintError(ERROR_IGNORE,TRUE);
			result = file.SkipStructure();
			break;
		case TAG_RIN:
			file.PrintError(ERROR_IGNORE,TRUE);
			result = file.NextLine();
			break;
		default:
			result = file.IllegalTag();
			break;
		}
		if (!result)
			return FALSE;
		if (!file.InterpretLine())
			return FALSE;
	}
	return TRUE;
}

BOOL GC_NOTE::ImportNote(GEDCOMFILE& file)
{
	// Note! Remember to check for structured notes!
	int TopLevel = file.GetLevel();
	if (file.GetPointer()[0])
	{
		StrCpy(NoteID,file.GetPointer());
		file.NextLine();
		return TRUE;
	}
	Note = file.GetText();
	file.NextLine();
	if (!file.InterpretLine())
		return (FALSE);
	while (file.GetLevel()>TopLevel)
	{
		switch(file.GetTag())
		{
		case TAG_CONT:
			Note.Add(TEXT("\r\n"));
			Note.Add(file.GetText());
			file.NextLine();
			break;
		case TAG_CONC:
			Note.Add(file.GetText());
			file.NextLine();
			break;
		case TAG_SOUR:
			file.PrintError(ERROR_IGNORE,TRUE);
			if (!file.SkipStructure())
				return FALSE;
			break;
		default:
			file.IllegalTag();
			break;
		}
		if (!file.InterpretLine())
			return (FALSE);
	}
	return (TRUE);
}

BOOL GC_NOTE::ImportInfo(GEDCOMFILE& file,VECTOR<GC_SOURCE*>& Sources,VECTOR<GC_SOURCECITE*>& Sourcerefs)
{
	GC_SOURCECITE* Sourceref;
	BOOL result = TRUE;

	// Note! Remember to check for structured notes!
	int TopLevel = file.GetLevel();
	if (file.GetPointer()[0])
	{
		StrCpy(NoteID,file.GetPointer());
		file.NextLine();
		return TRUE;
	}
	Note = file.GetText();
	file.NextLine();
	if (!file.InterpretLine())
		return (FALSE);
	while (file.GetLevel()>TopLevel)
	{
		switch(file.GetTag())
		{
		case TAG_CONT:
			Note.Add(TEXT("\r\n"));
			Note.Add(file.GetText());
			file.NextLine();
			break;
		case TAG_CONC:
			Note.Add(file.GetText());
			file.NextLine();
			break;
		case TAG_SOUR:
			Sourceref = new GC_SOURCECITE();
			result = Sourceref->Import(file,Sources);
			if (result)
				SourcerefIDs.Add(Sourcerefs.Add(Sourceref));
			else
			{
				delete Sourceref;
				return FALSE;
			}
			break;
		default:
			file.IllegalTag();
			break;
		}
		if (!file.InterpretLine())
			return (FALSE);
	}
	return (TRUE);
}

BOOL GC_STRING::Import(GEDCOMFILE& file)
{
	int TopLevel = file.GetLevel();
	Text = file.GetText();
	file.NextLine();
	if (!file.InterpretLine())
		return (FALSE);
	while (file.GetLevel()>TopLevel)
	{
		switch(file.GetTag())
		{
		case TAG_CONT:
			Text.Add(TEXT("\n"));
			Text.Add(file.GetText());
			file.NextLine();
			break;
		case TAG_CONC:
			Text.Add(file.GetText());
			file.NextLine();
			break;
		default:
			file.IllegalTag();
			break;
		}
		if (!file.InterpretLine())
			return (FALSE);
	}
	return (TRUE);
}