/****************************************************************
 *                                                              *
 *  Copyright (C) 2003-2009 Christoffer Owe                     *
 *  For conditions of distribution and use, see License.txt     *
 *                                                              *
 ****************************************************************/

#include <windows.h>
#include <strsafe.h>
#include "ged2xml.h"
#include "gc_person.h"

BOOL GC_PERSON::Import(GEDCOMFILE& file,VECTOR<EVENTTYPE*>& et,VECTOR<GC_SOURCE*>& Sources,VECTOR<GC_SOURCECITE*>& Sourcerefs,VECTOR<GC_TODO*>& Todos)
{
	GC_NAME* Name;
	GC_EVENT* Event;
	GC_ATTRIBUTE* Attribute;
	GC_NOTE* Note;
	GC_OBJECT* Object;
	GC_SOURCECITE* Sourceref;
	GC_TODO* Todo;
	TCHAR PersonID[XREFSIZE+1];
	TCHAR *ptr;
	BOOL result = TRUE;

	StringCbCopy(PersonID,sizeof(PersonID),file.GetXRef());
	if (!file.NextLine())
		return FALSE;
	if (!file.InterpretLine())
		return FALSE;
	while (file.GetLevel()>0 && result==TRUE)
	{
		switch(file.GetTag())
		{
		case TAG_RESN:
		case TAG_RFN:
		case TAG_AFN:
		case TAG_REFN:
		case TAG_RIN:
			if (file.ConvData->PersonInfoStatus==0)
			{
				file.PrintError(ERROR_IGNORE,TRUE);
				//result = file.NextLine();
			}
			//else if (file.ConvData->PersonInfoStatus==1)
			//	result = file.NextLine();
			else if (file.ConvData->PersonInfoStatus==2)
			{
				if (Comments.Text.Size())
					Comments.Text.Add(TEXT("\r\n"));
				if (file.GetTag()==TAG_RFN)
					Comments.Text.Add(TEXT("Permanent record file number "));
				else if (file.GetTag()==TAG_AFN)
					Comments.Text.Add(TEXT("Ancestry file number "));
				else if (file.GetTag()==TAG_REFN)
					Comments.Text.Add(TEXT("User reference number "));
				else if (file.GetTag()==TAG_RIN)
					Comments.Text.Add(TEXT("Automated record ID "));
				Comments.Text.Add(file.GetText());
				Comments.Text.Add(TEXT("."));
			}
			result = file.NextLine();
			break;
		case TAG_ANCI:
		case TAG_DESI:
			file.PrintError(ERROR_IGNORE,TRUE);
			result = file.NextLine();
			break;
		case TAG_FAMC:
		case TAG_FAMS:
			result = file.SkipStructure();
			break;
		case TAG_NAME:
			Name = new GC_NAME;
			result = Name->Import(file,Sources,Sourcerefs);
			Names.Add(Name);
			break;
		case TAG_ALIA:
			ptr = file.GetPointer();
			if (ptr[0])
			{
				file.PrintError(ERROR_IGNORE,TRUE);
				result = file.NextLine();
			}
			else if (file.ConvData->AliasNames)
			{
				Name = new GC_NAME;
				result = Name->Import(file,Sources,Sourcerefs);
				Names.Add(Name);
			}
			else
				result = file.NextLine();
			break;
		case TAG_BIRT:
		case TAG_CHR:
		case TAG_DEAT:
		case TAG_BURI:
		case TAG_CREM:
		case TAG_ADOP:
		case TAG_BAPM:
		case TAG_BARM:
		case TAG_BASM:
		case TAG_BLES:
		case TAG_CHRA:
		case TAG_CONF:
		case TAG_FCOM:
		case TAG_ORDN:
		case TAG_NATU:
		case TAG_EMIG:
		case TAG_IMMI:
		case TAG_CENS:
		case TAG_PROB:
		case TAG_WILL:
		case TAG_GRAD:
		case TAG_RETI:
		case TAG_RESI:
		case TAG_EVEN:
		case TAG_BAPL:
		case TAG_CONL:
		case TAG_ENDL:
		case TAG_SLGC:
			Event = new GC_EVENT();
			result = Event->Import(file,et,Sources,Sourcerefs,1);
			Events.Add(Event);
			break;
		case TAG_CAST:
		case TAG_DSCR:
		case TAG_EDUC:
		case TAG_IDNO:
		case TAG_NATI:
		case TAG_NCHI:
		case TAG_NMR:
		case TAG_OCCU:
		case TAG_PROP:
		case TAG_RELI:
		case TAG_SSN:
		case TAG_TITL:
		case TAG_FACT:
		case UTAG_UID:	// Legacy
		case UTAG_EMAIL:	// Legacy
			Attribute = new GC_ATTRIBUTE();
			result = Attribute->Import(file,Sources,Sourcerefs);
			Attributes.Add(Attribute);
			break;
		case TAG_ADDR:
		case TAG_PHON:
			if (file.ConvData->UserDefinedTags && file.GetSystem()==SYSTEM_BROTHERSKEEPERS)
			{
				Event = new GC_EVENT();
				result = Event->ImportAddress(file,et);
				Events.Add(Event);
			}
			break;
		case TAG_SEX:
			ptr = file.GetText();
			if (ptr[0]=='M')
				Sex = 1;
			else if (ptr[0]=='F')
				Sex = 2;
			result = file.NextLine();
			break;
		case TAG_NOTE:
			Note = new GC_NOTE();
			result = Note->ImportInfo(file,Sources,Sourcerefs);
			Notes.Add(Note);
			break;
		case TAG_OBJE:
			Object = new GC_OBJECT();
			result = Object->Import(file);
			Objects.Add(Object);
			break;
		case UTAG_STAT:
			if (file.GetSystem()==SYSTEM_LEGACY)
			{
				Note = new GC_NOTE();
				Note->Note = file.GetText();
				Notes.Add(Note);
				result = file.NextLine();
			}
			else
				result = file.IllegalTag();
			break;
		case TAG_SOUR:
			Sourceref = new GC_SOURCECITE();
			result = Sourceref->Import(file,Sources);
			SourcerefIDs.Add(Sourcerefs.Add(Sourceref));
			break;
		case TAG_CHAN:
			result = Change.Import(file);
			break;
		case UTAG_TODO:
			if (file.ConvData->UserDefinedTags && file.GetSystem()==SYSTEM_FAMILYORIGINS)
			{
				Todo = new GC_TODO();
				result = Todo->Import(file,PersonID);
				Todos.Add(Todo);
			}
			else
				result = file.IllegalTag();
			break;
		default:
			result = file.IllegalTag();
			break;
		}
		if (!result)
			return FALSE;
		if (!file.InterpretLine())
			return FALSE;
	}
	return TRUE;
}

