/****************************************************************
 *                                                              *
 *  Copyright (C) 2003-2009 Christoffer Owe                     *
 *  For conditions of distribution and use, see License.txt     *
 *                                                              *
 ****************************************************************/

#include <Windows.h>
#include <Shlwapi.h>
#include "ged2xml.h"
#include "gc_source.h"
#include "gc_object.h"

BOOL GC_SOURCE::Import(GEDCOMFILE& file)
{
	GC_OBJECT* Object;
	BOOL result = TRUE;

	file.NextLine();
	if (!file.InterpretLine())
		return (FALSE);
	while (file.GetLevel()>0)
	{
		switch(file.GetTag())
		{
		case TAG_REFN:
		case TAG_RIN:
			file.PrintError(ERROR_IGNORE,TRUE);
			result = file.NextLine();
			break;
		case TAG_AUTH:
			result = Author.Import(file);
			break;
		case TAG_TITL:
			result = Title.Import(file);
			break;
		case TAG_ABBR:
			Abbreviation = file.GetText();
			result = file.NextLine();
			break;
		case TAG_PUBL:
			result = Publication.Import(file);
			break;
		case TAG_TEXT:
			result = Text.Import(file);
			break;
		case TAG_REPO:
			if (!file.GetPointer()[0])
			{
				file.PrintError(ERROR_XREFMISSING,TRUE);
				result = file.SkipStructure();
			}
			else
				result = Repref.Import(file);
			break;
		case TAG_NOTE:
			result = Note.ImportNote(file);
			break;
		case TAG_OBJE:
			Object = new GC_OBJECT();
			result = Object->Import(file);
			Objects.Add(Object);
			break;
		case TAG_CHAN:
			result = Change.Import(file);
			break;
		default:
			result = file.IllegalTag();
			break;
		}
		if (!result)
			return FALSE;
		if (!file.InterpretLine())
			return (FALSE);
	}
	return (TRUE);
}

BOOL GC_SOURCECITE::Import(GEDCOMFILE& file,VECTOR<GC_SOURCE*>& Sources)
{
	int DataLevel,TopLevel;
	BOOL result = TRUE;

	TopLevel = file.GetLevel();
	if (file.GetPointer()[0])
	{
		// Structured source
		StrCpy(SourceID,file.GetPointer());
		if (!file.NextLine())
			return FALSE;
		if (!file.InterpretLine())
			return FALSE;
		while (file.GetLevel()>TopLevel)
		{
			switch(file.GetTag())
			{
				case TAG_PAGE:
					Page = file.GetText();
					result = file.NextLine();
					break;
				case TAG_QUAY:
					Quality = file.GetText();
					result = file.NextLine();
					break;
				case TAG_DATA:
					DataLevel = file.GetLevel();
					if (!file.NextLine())
						return FALSE;
					if (!file.InterpretLine())
						return FALSE;
					while (file.GetLevel()>DataLevel)
					{
						if (file.GetTag()==TAG_DATE)
						{
							TextDate = file.GetText();
							/*file.PrintError(ERROR_IGNORE,TRUE);
							if (!file.SkipStructure())
								return FALSE;*/
							if (!file.NextLine())
								return FALSE;
						}
						else if (file.GetTag()==TAG_TEXT)
						{
							if (!Text.Import(file))
								return FALSE;
						}
						else
							file.IllegalTag();
						if (!file.InterpretLine())
							return FALSE;
					}
					break;
				case TAG_NOTE:
					result = Note.ImportNote(file);
					break;
				default:
					result = file.IllegalTag();
					break;
			}
			if (!result)
				return FALSE;
			if (!file.InterpretLine())
				return FALSE;
		}
	}
	else
	{
		// Unstructured source
		SourceID[0] = 0;
		do
		{
			if (file.GetTag()==TAG_CONT)
				SourceDescription.Text.Add(TEXT("\r\n"));
			SourceDescription.Text.Add(file.GetText());
			if (!file.NextLine())
				return FALSE;
			if (!file.InterpretLine())
				return FALSE;
		}
		while (file.GetTag()==TAG_CONT || file.GetTag()==TAG_CONC);
		while (file.GetLevel()>TopLevel)
		{
			switch(file.GetTag())
			{
				case TAG_PAGE:
					if (file.ConvData->UserDefinedTags && file.GetSystem()==SYSTEM_BROTHERSKEEPERS)
					{
						Page = file.GetText();
						result = file.NextLine();
					}
					else
						result = file.IllegalTag();
					break;
				case TAG_TEXT:
					result = Text.Import(file);
					break;
				case TAG_NOTE:
					result = Note.ImportNote(file);
					break;
				default:
					result = file.IllegalTag();
					break;
			}
			if (!result)
				return FALSE;
			if (!file.InterpretLine())
				return FALSE;
		}
	}
	return TRUE;
}