/****************************************************************
 *                                                              *
 *  Copyright (C) 2003-2007 Christoffer Owe                     *
 *  For conditions of distribution and use, see License.txt     *
 *                                                              *
 ****************************************************************/

#include <windows.h>
#include <Shlwapi.h>
#include <strsafe.h>
#include "gedcom.h"
#include "ged2xml.h"

TCHAR GedcomTags[]=
		TEXT("ABBR ADDR ADR1 ADR2 ADOP AFN  AGE  AGNC ALIA ANCE ")\
		TEXT("ANCI ANUL ASSO AUTH BAPL BAPM BARM BASM BIRT BLES ")\
		TEXT("BLOB BURI CALN CAST CAUS CENS CHAN CHAR CHIL CHR  ")\
		TEXT("CHRA CITY CONC CONF CONL CONT COPR CORP CREM CTRY ")\
		TEXT("DATA DATE DEAT DESC DESI DEST DIV  DIVF DSCR EDUC ")\
		TEXT("EMIG ENDL ENGA EVEN FAM  FAMC FAMF FAMS FCOM FILE ")\
		TEXT("FORM GEDC GIVN GRAD HEAD HUSB IDNO IMMI INDI LANG ")\
		TEXT("LEGA MARB MARC MARL MARR MARS MEDI NAME NATI NATU ")\
		TEXT("NCHI NICK NMR  NOTE NPFX NSFX OBJE OCCU ORDI ORDN ")\
		TEXT("PAGE PEDI PHON PLAC POST PROB PROP PUBL QUAY REFN ")\
		TEXT("RELA RELI REPO RESI RESN RETI RFN  RIN  ROLE SEX  ")\
		TEXT("SLGC SLGS SOUR SPFX SSN  STAE STAT SUBM SUBN SURN ")\
		TEXT("TEMP TEXT TIME TITL TRLR TYPE VERS WIFE WILL EMAIL")\
		TEXT("FACT FAX  FONE LATI LONG MAP  ROMN WWW  ");
TCHAR UserTags[]=TEXT("EMAILUID  STAT TODO ");


GEDCOMFILE::GEDCOMFILE(HANDLE hFile,CONVERSIONDATA* convdata)	//HANDLE LogFile,char* (*ErrMsg)(int ErrorID),int MsgLevel)
{
	ConvData = convdata;
	//this->LogFile = LogFile;
	//this->ErrMsg = ErrMsg;
	//this->MessageLevel = MsgLevel;
	FileSize = GetFileSize(hFile,NULL);
	FileMapping = CreateFileMapping(hFile,NULL,PAGE_READONLY,0,0,NULL);
	file = (char*) MapViewOfFile(FileMapping,FILE_MAP_READ,0,0,0);
	CurrentLine = 0;
	wCurrentLine = 0;
	EndOfLine = 0;
	wEndOfLine = 0;
	LineNumber = 0;
	Index = 0;
	szText[0] = 0;
	Tag = 0;
	XRef[0] = 0;
	Pointer[0] = 0;
	LastErrorLine = 0;
	// Check character set
	if ((BYTE) file[0]==0xEF && (BYTE) file[1]==0xBB && (BYTE) file[2]==0xBF)
	{
		CharacterSet = CHARSET_UTF8;
		EndOfLine += 3;
	}
	else if ((BYTE) file[0]==0xFF && (BYTE) file[1]==0xFE)
	{
		CharacterSet = CHARSET_UNICODE;
		wfile = (WCHAR*) file;
		wEndOfLine++;
	}
	else if ((BYTE) file[0]==0xFE && (BYTE) file[1]==0xFF)
	{
		CharacterSet = CHARSET_UCB;
		EndOfLine += 2;
	}
	else
		CharacterSet = CHARSET_ANSEL;
	System = SYSTEM_UNKNOWN;
}

GEDCOMFILE::~GEDCOMFILE()
{
	UnmapViewOfFile(file);
	CloseHandle(FileMapping);
}

BOOL GEDCOMFILE::NextLine()
{
	LineNumber++;
	// Update pointers
	if (CharacterSet==CHARSET_UNICODE)
	{
		wCurrentLine = wEndOfLine;
		while (wfile[wCurrentLine]=='\r' || wfile[wCurrentLine]=='\n')
		{
			wCurrentLine++;
			if (wCurrentLine*2>=FileSize)
				return TRUE;
		}
		wEndOfLine = wCurrentLine + 1;
		while (wEndOfLine*2<FileSize && wfile[wEndOfLine]!='\r' && wfile[wEndOfLine]!='\n')
			wEndOfLine++;
		// Check if file is finished
		if (wCurrentLine*2>=FileSize)
		{
			PrintError(ERROR_EOF,TRUE);
			return (FALSE);
		}
	}
	else
	{
		CurrentLine = EndOfLine;
		while (file[CurrentLine]=='\r' || file[CurrentLine]=='\n')
		{
			CurrentLine++;
			if (CurrentLine>=FileSize)
				return TRUE;
		}
		EndOfLine = CurrentLine + 1;
		while (EndOfLine<FileSize && file[EndOfLine]!='\r' && file[EndOfLine]!='\n')
			EndOfLine++;
		// Check if file is finished
		if (CurrentLine>=FileSize)
		{
			PrintError(ERROR_EOF,TRUE);
			return (FALSE);
		}
	}
	// Copy and translate line
	if ((EndOfLine-CurrentLine>TEXTSIZE-1 && CharacterSet==CHARSET_UNICODE) ||
		(wEndOfLine-wCurrentLine>TEXTSIZE-1 && CharacterSet!=CHARSET_UNICODE))
	{
		PrintError(ERROR_LINE,TRUE);
		return FALSE;	// Fatal error
	}
	if (CharacterSet==CHARSET_ANSI)
	{
		TranslateANSI(szText,file+CurrentLine,EndOfLine-CurrentLine);
		//strncpy(szText,file+CurrentLine,EndOfLine-CurrentLine);
		//szText[EndOfLine-CurrentLine] = 0;
	}
	else if (CharacterSet==CHARSET_IBMPC)
		TranslateIBMPC(szText,file+CurrentLine,EndOfLine-CurrentLine);
	else if (CharacterSet==CHARSET_UNICODE)
	{
		StringCbCopyN(szText,sizeof(szText),wfile+wCurrentLine,wEndOfLine-wCurrentLine);
		szText[wEndOfLine-wCurrentLine] = 0;
	}
	else if (CharacterSet==CHARSET_UTF8)
		TranslateUTF8(szText,file+CurrentLine,EndOfLine-CurrentLine);
	else
		TranslateANSEL(szText,file+CurrentLine,EndOfLine-CurrentLine);
	// Get level
	Index = 0;
	if (!isdigit(szText[Index]))
	{
		PrintError(ERROR_LINE,TRUE);
		return FALSE;	// Fatal error
	}
	Level = szText[Index]-0x30;
	Index++;
	if (szText[Index]!=' ')
	{
		PrintError(ERROR_LINE,TRUE);
		return FALSE;	// Fatal error
	}
	Index++;
	return TRUE;
}

void GEDCOMFILE::TranslateANSEL(TCHAR* szTo,char* szFrom,int nCharacters)
{
	static unsigned char FromANSEL[63]={76,216,208,222,198,32,145,183,32,174,177,79,85,39,32,
		96,108,248,100,254,230,32,34,105,163,240,208,111,117,32,32,
		176,108,32,169,32,191,161,32,32,32,32,32,32,32,32,223,
		32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32};
	//unsigned char* szToPtr = (unsigned char*) szTo;
	TCHAR* szToPtr = szTo;
	unsigned char* szFromPtr = (unsigned char*) szFrom;
	int i,j;
	unsigned int chr,chr2;

	i=0;
	j=0;
	while (i<nCharacters)
	{
		chr = szFromPtr[i];
		if (chr<128)
		{
			szToPtr[j] = szFromPtr[i];
			j++;
			i++;
		}
		else if (chr<161) i++;
		else if (chr<225)
		{
			szToPtr[j] = FromANSEL[chr-161];
			j++;
			i++;
		}
		else
		{
			i++;
			chr2 = szFromPtr[i];
			i++;
			switch (chr)
			{
				case 225:     //grave accent
					switch (chr2)
					{
						case 'a':
							szToPtr[j] = 224;
							break;
						case 'A':
							szToPtr[j] = 192;
							break;
						case 'e':
							szToPtr[j] = 232;
							break;
						case 'E':
							szToPtr[j] = 200;
							break;
						case 'i':
							szToPtr[j] = 236;
							break;
						case 'I':
							szToPtr[j] = 204;
							break;
						case 'o':
							szToPtr[j] = 242;
							break;
						case 'O':
							szToPtr[j] = 210;
							break;
						case 'u':
							szToPtr[j] = 249;
							break;
						case 'U':
							szToPtr[j] = 217;
							break;
						default:
							if (chr2<128) szToPtr[j] = chr2;
							else j--;
							break;
					}
					break;
				case 226:     //acute accent
					switch (chr2)
					{
						case 'a':
							szToPtr[j] = 225;
							break;
						case 'A':
							szToPtr[j] = 193;
							break;
						case 'e':
							szToPtr[j] = 233;
							break;
						case 'E':
							szToPtr[j] = 201;
							break;
						case 'i':
							szToPtr[j] = 237;
							break;
						case 'I':
							szToPtr[j] = 205;
							break;
						case 'o':
							szToPtr[j] = 243;
							break;
						case 'O':
							szToPtr[j] = 211;
							break;
						case 'u':
							szToPtr[j] = 250;
							break;
						case 'U':
							szToPtr[j] = 218;
							break;
						case 'y':
							szToPtr[j] = 254;
							break;
						case 'Y':
							szToPtr[j] = 222;
							break;
						default:
							if (chr2<128) szToPtr[j] = chr2;
							else j--;
							break;
					}
					break;
				case 227:     //circumflex accent
					switch (chr2)
					{
						case 'a':
							szToPtr[j] = 226;
							break;
						case 'A':
							szToPtr[j] = 194;
							break;
						case 'e':
							szToPtr[j] = 234;
							break;
						case 'E':
							szToPtr[j] = 202;
							break;
						case 'i':
							szToPtr[j] = 238;
							break;
						case 'I':
							szToPtr[j] = 206;
							break;
						case 'o':
							szToPtr[j] = 244;
							break;
						case 'O':
							szToPtr[j] = 212;
							break;
						case 'u':
							szToPtr[j] = 251;
							break;
						case 'U':
							szToPtr[j] = 219;
							break;
						default:
							if (chr2<128) szToPtr[j] = chr2;
							else j--;
							break;
					}
					break;
				case 228:     //tilde
					switch (chr2)
					{
						case 'a':
							szToPtr[j] = 227;
							break;
						case 'A':
							szToPtr[j] = 195;
							break;
						case 'o':
							szToPtr[j] = 245;
							break;
						case 'O':
							szToPtr[j] = 213;
							break;
						case 'n':
							szToPtr[j] = 241;
							break;
						case 'N':
							szToPtr[j] = 209;
							break;
						default:
							if (chr2<128) szToPtr[j] = chr2;
							else j--;
							break;
					}
					break;
				case 232:     //umlaut
					switch (chr2)
					{
						case 'a':
							szToPtr[j] = 228;
							break;
						case 'A':
							szToPtr[j] = 196;
							break;
						case 'e':
							szToPtr[j] = 235;
							break;
						case 'E':
							szToPtr[j] = 203;
							break;
						case 'i':
							szToPtr[j] = 239;
							break;
						case 'I':
							szToPtr[j] = 207;
							break;
						case 'o':
							szToPtr[j] = 246;
							break;
						case 'O':
							szToPtr[j] = 214;
							break;
						case 'u':
							szToPtr[j] = 252;
							break;
						case 'U':
							szToPtr[j] = 220;
							break;
						case 'y':
							szToPtr[j] = 255;
							break;
						default:
							if (chr2<128) szToPtr[j] = chr2;
							else j--;
							break;
					}
					break;
				case 234:     //circle above
					switch (chr2)
					{
						case 'a':
							szToPtr[j] = 229;
							break;
						case 'A':
							szToPtr[j] = 197;
							break;
						default:
							if (chr2<128) szToPtr[j] = chr2;
							else j--;
							break;
					}
					break;
				case 240:     //cedilla
					switch (chr2)
					{
						case 'c':
							szToPtr[j] = 231;
							break;
						case 'C':
							szToPtr[j] = 199;
							break;
						default:
							if (chr2<128) szToPtr[j] = chr2;
							else j--;
							break;
					}
					break;
				case 246:     //underscore
					switch (chr2)
					{
						case 'a':
							szToPtr[j] = 170;
							break;
						case 'o':
							szToPtr[j] = 186;
							break;
						default:
							if (chr2<128) szToPtr[j] = chr2;
							else j--;
							break;
					}
					break;
				default:
					if (chr2<128) szToPtr[j] = chr2;
					else j--;
					break;
			}
			j++;
		}
	}
	szToPtr[j] = 0;
}

void GEDCOMFILE::TranslateIBMPC(TCHAR* szTo,char* szFrom,int nCharacters)
{
	static unsigned char FromIBMPC[128]={199,252,233,226,228,224,229,231,234,235,232,239,238,236,196,197,
		201,230,198,244,246,242,251,249,255,214,220,248,163,216,80,102,
		225,237,243,250,241,209,170,32,191,32,172,189,188,161,171,187,
		32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
		32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
		32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,		
		32,223,32,32,32,32,181,222,254,32,32,32,32,32,32,32,
		61,177,32,32,32,32,247,32,176,176,183,32,32,178,32,32};

	for (int i=0;i<nCharacters;i++)
	{
		if (szFrom[i]<128)
			szTo[i] = szFrom[i];
		else
			szTo[i] = FromIBMPC[szTo[i]-128];
	}
	szTo[i] = 0;
}

void GEDCOMFILE::TranslateUTF8(TCHAR* szTo,char* szFrom,int nCharacters)
{
	//LPWSTR tmp = (LPWSTR) HeapAlloc(GetProcessHeap(),HEAP_ZERO_MEMORY,nCharacters*2+8);
	//MultiByteToWideChar(CP_UTF8,0,szFrom,nCharacters,tmp,nCharacters);
	//int nBytes = WideCharToMultiByte(CP_ACP,0,tmp,nCharacters,szTo,nCharacters,NULL,NULL);
	//szTo[nBytes] = 0;
	//HeapFree(GetProcessHeap(),0,tmp);
	int cch = MultiByteToWideChar(CP_UTF8,0,szFrom,nCharacters,szTo,nCharacters);
	szTo[cch] = 0;
}

/*void GEDCOMFILE::TranslateUnicode(TCHAR* szTo,LPWSTR szFrom,int nCharacters)
{
	int nBytes = WideCharToMultiByte(CP_ACP,0,szFrom,nCharacters,szTo,nCharacters,NULL,NULL);
	szTo[nBytes] = 0;
}*/

void GEDCOMFILE::TranslateANSI(TCHAR* szTo,char* szFrom,int nCharacters)
{
	int nBytes = MultiByteToWideChar(CP_ACP,0,szFrom,nCharacters,szTo,nCharacters);
	szTo[nBytes] = 0;
}

BOOL GEDCOMFILE::InterpretLine()
{
	int x,i;

start:
	Index = 2;
	Tag = 0;
	XRef[0] = 0;
	Pointer[0] = 0;
	// Get XRef
	if (szText[Index]=='@')
	{
		TranslatePointer(XRef);
		if (!XRef[0])
		{
			PrintError(ERROR_XREFTOOLONG,TRUE);
			return FALSE;	// Fatal error
		}
		if (szText[Index]!=' ')
		{
			PrintError(ERROR_LINE,TRUE);
			return FALSE;	// Fatal error
		}
		Index++;
	}
	// Get tag
	if (szText[Index]=='_')
	{
		if (ConvData->UserDefinedTags)
		{
			Index++;
			StrCpyN(szTag,&szText[Index],5);
			szTag[5] = 0;
			if (!szTag[3])
				szTag[3] = ' ';
			if (!szTag[4] || szTag[3]==' ')
				szTag[4] = ' ';
			for (i=UTAG_USERSTART;i<=UTAG_USERDEFINED;i++)
			{
				x = (i-UTAG_USERSTART)*5;
				if (!StrCmpN(szTag,&UserTags[x],5))
					break;
			}
			if (i>UTAG_USERDEFINED)
			{
				PrintError(ERROR_UNKNOWNTAG,TRUE);
				if (!SkipStructure())
					return FALSE;
				goto start;
			}
			Index += 3;
			if (szTag[3]!=' ') Index++;
			if (szTag[4]!=' ') Index++;
		}
		else
		{
			PrintError(ERROR_USERDEFINED,TRUE);
			if (!SkipStructure())
				return FALSE;
			goto start;
		}
	}
	else
	{
		StrCpyN(szTag,&szText[Index],5);
		szTag[5] = 0;
		if (!szTag[3])
			szTag[3] = ' ';
		if (!szTag[4] || szTag[3]==' ')
			szTag[4] = ' ';
		for (i=1;i<=TAG_GEDMAX;i++)
		{
			x = (i-1)*5;
			if (!StrCmpN(szTag,&GedcomTags[x],5))
				break;

		}
		if (i>TAG_GEDMAX)
		{
			PrintError(ERROR_UNKNOWNTAG,TRUE);
			if (!NextLine())
				return FALSE;
			goto start;
		}
		Index += 3;
		if (szTag[3]!=' ') Index++;
		if (szTag[4]!=' ') Index++;
	}
	while  (szText[Index]==' ')
		Index++;
	Tag = i;
	// Get pointer
	if (szText[Index]=='@' && szText[Index+1]!='@')
	{
		TranslatePointer(Pointer);
		if (!Pointer[0])
		{
			PrintError(ERROR_XREFTOOLONG,TRUE);
			return FALSE;	// Fatal error
		}
		if (szText[Index] && szText[Index]!=' ')
		{
			PrintError(ERROR_LINE,TRUE);
			return FALSE;	// Fatal error
		}
		Index++;
	}
	return TRUE;
}

void GEDCOMFILE::TranslatePointer(TCHAR* result)
{
	//long id;
	int x;

	Index++;		// Skipping the first '@'
	x = 0;
	while (x<XREFSIZE && szText[Index+x]!='@')
	{
		result[x] = szText[Index+x];
		x++;
	}
	if (x>=XREFSIZE)	// XREF too long?
	{
		result[0] = 0;
		return;
	}
	result[x] = 0;
	Index += x;
	Index++;
	/*if (isalpha(file[CurrentLine+Index]))
		Index++;
	x = Index;
	while (file[CurrentLine+x]!='@')
		x++;
	if (x-Index>5)
		return -1;	// id too long
	id = atol(file+CurrentLine+Index);
	if (!id && (file[CurrentLine+Index]!='0' || x-Index>1))
		return -1;	// alphanumeric id
	Index = x + 1;
	return id;*/
}

void GEDCOMFILE::WriteLogFile(TCHAR* szText)
{
	DWORD BytesWritten;
	DWORD CharactersWritten;

	if (ConvData->LogFileIsConsole)
		//WriteConsole(ConvData->hLogFile,szText,lstrlen(szText),&CharactersWritten,NULL);
		wprintf(szText);
	else
		WriteFile(ConvData->hLogFile,szText,lstrlen(szText)*sizeof(TCHAR),&BytesWritten,NULL);
}

void GEDCOMFILE::PrintError(int Error,BOOL ShowLine)
{
	TCHAR buffer[64];
	//DWORD BytesWritten;
	TCHAR* Message;

	if (LineNumber==LastErrorLine)
		return;	// Don't display more than one error on a line
	LastErrorLine = LineNumber;
	// Check for known errors
	if (Error==ERROR_UNKNOWNTAG && ConvData->MessageLevel>=MESSAGELEVEL_SKIPERRORS)
	{
		if (!StrCmp(szTag,TEXT("NUMB")) ||
			!StrCmp(szTag,TEXT("LVG ")) ||
			!StrCmp(szTag,TEXT("ILLE"))) // Known ROOTSIII errors
			return;
	}
	if (Error==ERROR_IGNORE && ConvData->MessageLevel>=MESSAGELEVEL_SKIPIGNORE)
		return;
	// Process error
	Message = ConvData->MsgProvider(Error);
	if (Error==ERROR_IGNORE)
		StringCbPrintf(buffer,sizeof(buffer),TEXT("\r\nLine %i: "),LineNumber);
	else
		StringCbPrintf(buffer,sizeof(buffer),TEXT("\r\nError on line %i: "),LineNumber);
	//WriteFile(ConvData->hLogFile,buffer,lstrlen(buffer)*sizeof(TCHAR),&BytesWritten,NULL);
	WriteLogFile(buffer);
	//WriteFile(ConvData->hLogFile,Message,lstrlen(Message)*sizeof(TCHAR),&BytesWritten,NULL);
	WriteLogFile(Message);
	//WriteFile(ConvData->hLogFile,TEXT("\r\n"),2,&BytesWritten,NULL);
	WriteLogFile(TEXT("\r\n"));
	if (ShowLine)
	{
		/*int length = EndOfLine-CurrentLine;
		if (length>80)
			length = 80;*/
		//WriteFile(ConvData->hLogFile,szText,lstrlen(szText),&BytesWritten,NULL);
		WriteLogFile(szText);
		StringCbCopy(buffer,sizeof(buffer),TEXT("\r\n"));
		for	(int i=0;i<Index;i++)
			StringCbCat(buffer,sizeof(buffer),TEXT(" "));
		StringCbCat(buffer,sizeof(buffer),TEXT("^\r\n"));
		//WriteFile(ConvData->hLogFile,buffer,lstrlen(buffer),&BytesWritten,NULL);
		WriteLogFile(buffer);
	}
	//WriteFile(LogFile,"\r\n",2,&BytesWritten,NULL);
}

BOOL GEDCOMFILE::SkipStructure()
{
	static TCHAR szSkip[]=TEXT("The following lines were skipped:\r\n");
	int TopLevel,length;
	//DWORD BytesWritten;

	TopLevel = Level;
	if (!NextLine())
		return FALSE;
	//if (!CheckSyntax())
	//	return FALSE;
	if (Level<=TopLevel)
		return TRUE;	// No lines to skip
	//WriteFile(ConvData->hLogFile,szSkip,lstrlen(szSkip)*sizeof(TCHAR),&BytesWritten,NULL);
	WriteLogFile(szSkip);
	while (Level>TopLevel)
	{
		/*length = EndOfLine-CurrentLine;
		if (length>80)
			length = 80;
		WriteFile(ConvData->hLogFile,&file[CurrentLine],length,&BytesWritten,NULL);*/
		length = lstrlen(szText);
		if (length>80)
			length = 80;
		//WriteFile(ConvData->hLogFile,szText,length*sizeof(TCHAR),&BytesWritten,NULL);
		WriteLogFile(szText);
		//WriteFile(ConvData->hLogFile,TEXT("\r\n"),2*sizeof(TCHAR),&BytesWritten,NULL);
		WriteLogFile(TEXT("\r\n"));
		if (!NextLine())
			return FALSE;
	}
	//WriteFile(ConvData->hLogFile,TEXT("\r\n"),2*sizeof(TCHAR),&BytesWritten,NULL);
	WriteLogFile(TEXT("\r\n"));
	return TRUE;
}

BOOL GEDCOMFILE::IllegalTag()
{
	Index = 2;
	PrintError(ERROR_ILLEGALTAG,TRUE);
	return SkipStructure();
}