/****************************************************************
 *                                                              *
 *  Copyright (C) 2003,2005 Christoffer Owe                     *
 *  For conditions of distribution and use, see License.txt     *
 *                                                              *
 ****************************************************************/

#ifndef __GEDCOM
#define __GEDCOM

#include <windows.h>
#include "common.h"
#include "ged2xml.h"

// Maximum size of text in gedcom lines
#define TEXTSIZE	256

// GEDCOM tags
#define TAG_ABBR	1
#define TAG_ADDR	2
#define TAG_ADR1	3
#define TAG_ADR2	4
#define TAG_ADOP	5
#define TAG_AFN		6
#define TAG_AGE		7
#define TAG_AGNC	8
#define TAG_ALIA	9
#define TAG_ANCE	10
#define TAG_ANCI	11
#define TAG_ANUL	12
#define TAG_ASSO	13
#define TAG_AUTH	14
#define TAG_BAPL	15
#define TAG_BAPM	16
#define TAG_BARM	17
#define TAG_BASM	18
#define TAG_BIRT	19
#define TAG_BLES	20
#define TAG_BLOB	21
#define TAG_BURI	22
#define TAG_CALN	23
#define TAG_CAST	24
#define TAG_CAUS	25
#define TAG_CENS	26
#define TAG_CHAN	27
#define TAG_CHAR	28
#define TAG_CHIL	29
#define TAG_CHR		30
#define TAG_CHRA	31
#define TAG_CITY	32
#define TAG_CONC	33
#define TAG_CONF	34
#define TAG_CONL	35
#define TAG_CONT	36
#define TAG_COPR	37
#define TAG_CORP	38
#define TAG_CREM	39
#define TAG_CTRY	40
#define TAG_DATA	41
#define TAG_DATE	42
#define TAG_DEAT	43
#define TAG_DESC	44
#define TAG_DESI	45
#define TAG_DEST	46
#define TAG_DIV		47
#define TAG_DIVF	48
#define TAG_DSCR	49
#define TAG_EDUC	50
#define TAG_EMIG	51
#define TAG_ENDL	52
#define TAG_ENGA	53
#define TAG_EVEN	54
#define TAG_FAM		55
#define TAG_FAMC	56
#define TAG_FAMF	57
#define TAG_FAMS	58
#define TAG_FCOM	59
#define TAG_FILE	60
#define TAG_FORM	61
#define TAG_GEDC	62
#define TAG_GIVN	63
#define TAG_GRAD	64
#define TAG_HEAD	65
#define TAG_HUSB	66
#define TAG_IDNO	67
#define TAG_IMMI	68
#define TAG_INDI	69
#define TAG_LANG	70
#define TAG_LEGA	71
#define TAG_MARB	72
#define TAG_MARC	73
#define TAG_MARL	74
#define TAG_MARR	75
#define TAG_MARS	76
#define TAG_MEDI	77
#define TAG_NAME	78
#define TAG_NATI	79
#define TAG_NATU	80
#define TAG_NCHI	81
#define TAG_NICK	82
#define TAG_NMR		83
#define TAG_NOTE	84
#define TAG_NPFX	85
#define TAG_NSFX	86
#define TAG_OBJE	87
#define TAG_OCCU	88
#define TAG_ORDI	89
#define TAG_ORDN	90
#define TAG_PAGE	91
#define TAG_PEDI	92
#define TAG_PHON	93
#define TAG_PLAC	94
#define TAG_POST	95
#define TAG_PROB	96
#define TAG_PROP	97
#define TAG_PUBL	98
#define TAG_QUAY	99
#define TAG_REFN	100
#define TAG_RELA	101
#define TAG_RELI	102
#define TAG_REPO	103
#define TAG_RESI	104
#define TAG_RESN	105
#define TAG_RETI	106
#define TAG_RFN		107
#define TAG_RIN		108
#define TAG_ROLE	109
#define TAG_SEX		110
#define TAG_SLGC	111
#define TAG_SLGS	112
#define TAG_SOUR	113
#define TAG_SPFX	114
#define TAG_SSN		115
#define TAG_STAE	116
#define TAG_STAT	117
#define TAG_SUBM	118
#define TAG_SUBN	119
#define TAG_SURN	120
#define TAG_TEMP	121
#define TAG_TEXT	122
#define TAG_TIME	123
#define TAG_TITL	124
#define TAG_TRLR	125
#define TAG_TYPE	126
#define TAG_VERS	127
#define TAG_WIFE	128
#define TAG_WILL	129
#define TAG_EMAIL	130	// Gedcom 5.5.1
#define TAG_FACT	131	// Gedcom 5.5.1
#define TAG_FAX		132 // Gedcom 5.5.1
#define TAG_FONE	133	// Gedcom 5.5.1 (unsupported)
#define TAG_LATI	134	// Gedcom 5.5.1 (unsupported)
#define TAG_LONG	135	// Gedcom 5.5.1 (unsupported)
#define TAG_MAP		136	// Gedcom 5.5.1 (unsupported)
#define TAG_ROMN	137	// Gedcom 5.5.1 (unsupported)
#define TAG_WWW		138	// Gedcom 5.5.1
#define TAG_GEDMAX	138
// User-defined tags
#define UTAG_USERSTART		1001
#define UTAG_EMAIL			1001
#define UTAG_UID			1002
#define UTAG_STAT			1003
#define UTAG_TODO			1004
#define UTAG_USERDEFINED	1004

// Constants for CharacterSet
#define CHARSET_ANSI	0
#define CHARSET_ANSEL	1
#define CHARSET_IBMPC	2
#define CHARSET_UTF8	3
#define CHARSET_UNICODE	4	// Unicode little-endian
#define CHARSET_UCB		5	// Unicode big-endian

// Constants for system
#define SYSTEM_UNKNOWN			0
#define SYSTEM_LEGACY			1
#define SYSTEM_FAMILYORIGINS	2
#define SYSTEM_BROTHERSKEEPERS	3

extern TCHAR GedcomTags[];

class GEDCOMFILE
{
private:
	//char* (*ErrMsg)(int ErrorID);
	//HANDLE LogFile;
	HANDLE FileMapping;
	//int MessageLevel;
	TCHAR szText[TEXTSIZE];
	TCHAR szTag[6];
	char* file;
	WCHAR* wfile;
	long FileSize;
	long CurrentLine;
	long wCurrentLine;
	long EndOfLine;
	long wEndOfLine;
	int LineNumber;
	int Index;
	int Tag;
	int Level;
	TCHAR XRef[XREFSIZE+1];
	TCHAR Pointer[XREFSIZE+1];
	int LastErrorLine;	// Last line on which there was an error
	BYTE CharacterSet;
	BYTE System;

	//BOOL CheckSyntax();
	void TranslatePointer(TCHAR* result);
	void TranslateANSEL(TCHAR* szTo,char* szFrom,int nCharacters);
	void TranslateIBMPC(TCHAR* szTo,char* szFrom,int nCharacters);
	void TranslateUTF8(TCHAR* szTo,char* szFrom,int nCharacters);
	//void TranslateUnicode(TCHAR* szTo,LPWSTR szFrom,int nCharacters);
	void TranslateANSI(TCHAR* szTo,char* szFrom,int nCharacters);
	void WriteLogFile(TCHAR* szText);

public:
	GEDCOMFILE(HANDLE GedcomFile,CONVERSIONDATA* ConvData);
	~GEDCOMFILE();
	BOOL NextLine();
	BOOL InterpretLine();
	void PrintError(int Error,BOOL ShowLine);
	BOOL SkipStructure();
	BOOL IllegalTag();	// "Default" for all import routines
	inline int GetLevel(){return Level;}
	inline int GetTag(){return Tag;}
	inline TCHAR* GetXRef(){return XRef;}
	inline TCHAR* GetPointer(){return Pointer;}
	inline TCHAR* GetText(){return &szText[Index];}
	inline int GetLineNumber(){return LineNumber;}
	//inline int GetIndex(){return Index;}
	inline void AddIndex(int x){Index+=x;}
	inline BOOL GetMsgLevel(){return ConvData->MessageLevel;}
	inline long GetGedcomFileSize(){return FileSize;}
	inline long GetDataRead(){return CurrentLine;}
	inline BYTE GetCharacterSet(){return CharacterSet;}
	inline void SetCharacterSet(BYTE CharSet){CharacterSet=CharSet;}
	inline BYTE GetSystem(){return System;}
	inline void SetSystem(BYTE system){System=system;}
	//inline void ResetIndex(){Index=0;}			// Use InterpretLine() instead
	CONVERSIONDATA* ConvData;
};

#endif