/****************************************************************
 *                                                              *
 *  Copyright (C) 2003-2009 Christoffer Owe                     *
 *  For conditions of distribution and use, see License.txt     *
 *                                                              *
 ****************************************************************/

#include <windows.h>
#include <Shlwapi.h>
#include <StrSafe.h>
#include "common.h"

STRING::STRING()
{
	size = 0;
	MaxSize = 0;
	ptr = NULL;
}

STRING::STRING(TCHAR* text)
{
	size = lstrlen(text);
	MaxSize = size + size/10 + 8;
	ptr = (TCHAR*) HeapAlloc(GetProcessHeap(),HEAP_ZERO_MEMORY,MaxSize*sizeof(TCHAR));
/*#ifdef _DEBUG
	DWORD size = GlobalSize(obj);
	if (obj==NULL)
		MessageBox(NULL,"Null-pointer from GlobalAlloc()","",0);
#endif*/
	StringCchCopy(ptr,MaxSize,text);
}

STRING::~STRING()
{
	if (ptr)
		HeapFree(GetProcessHeap(),0,ptr);
}

TCHAR* STRING::Get()
{
	if (ptr)
		return ptr;
	else return TEXT("");
}

long STRING::Size()
{
	return size;
}

void STRING::SetSize(long length)
{
	if (length>MaxSize)
	{
		MaxSize = length;
		if (ptr)
			ptr = (TCHAR*) HeapReAlloc(GetProcessHeap(),HEAP_ZERO_MEMORY,ptr,MaxSize*sizeof(TCHAR));
		else
			ptr = (TCHAR*) HeapAlloc(GetProcessHeap(),HEAP_ZERO_MEMORY,MaxSize*sizeof(TCHAR));
	}
}

/*void STRING::Copy(char* buffer)
{
	char* string = (char*) GlobalLock(obj);
	strcpy(buffer,string);
}*/

void STRING::Add(STRING &string)
{
	size += string.size;
	if (size>MaxSize-1)
	{
		MaxSize += string.MaxSize;
		if (ptr)
			ptr = (TCHAR*) HeapReAlloc(GetProcessHeap(),HEAP_ZERO_MEMORY,ptr,MaxSize*sizeof(TCHAR));
		else
			ptr = (TCHAR*) HeapAlloc(GetProcessHeap(),HEAP_ZERO_MEMORY,MaxSize*sizeof(TCHAR));
	}
	StringCchCat(ptr,MaxSize,string.ptr);
	//length = strlen(ptr);
}

void STRING::Add(const TCHAR* string)
{
	Add(string,lstrlen(string));
}

void STRING::Add(const TCHAR* string,int length)
{
	size += length;
	if (size>MaxSize-1)
	{
		MaxSize = size + size/10 + 8;
		if (ptr)
			ptr = (TCHAR*) HeapReAlloc(GetProcessHeap(),HEAP_ZERO_MEMORY,ptr,MaxSize*sizeof(TCHAR));
		else
			ptr = (TCHAR*) HeapAlloc(GetProcessHeap(),HEAP_ZERO_MEMORY,MaxSize*sizeof(TCHAR));
	}
	StringCchCatN(ptr,MaxSize,string,length);
	ptr[size] = 0;
}

void STRING::Set(const TCHAR* text)
{
	if (ptr)
		HeapFree(GetProcessHeap(),0,ptr);
	size = lstrlen(text);
	MaxSize = size + size/10 + 8;
	ptr = (TCHAR*) HeapAlloc(GetProcessHeap(),HEAP_ZERO_MEMORY,MaxSize*sizeof(TCHAR));
	StringCchCopy(ptr,MaxSize,text);
}

void STRING::Set(const TCHAR* text,int length)
{
	if (ptr)
		HeapFree(GetProcessHeap(),0,ptr);
	size = min(lstrlen(text),length);
	MaxSize = size + size/10 + 8;
	ptr = (TCHAR*) HeapAlloc(GetProcessHeap(),HEAP_ZERO_MEMORY,MaxSize*sizeof(TCHAR));
	StringCchCopyN(ptr,MaxSize,text,length);
	//ptr[size] = 0;
}

//Returns the length of the string
int STRING::Get(TCHAR* text,int length)
{
	if (length)
	{
		StringCbCopy(text,length,ptr);
		//int l = min(size,length-1);
		//strncpy(text,ptr,l);
		//text[l] = 0;
	}
	return size;
}

// Converts the string to a valid character data XML string.
void STRING::ConvertToXML(STRING &text)
{
	if (!size) return;
	text.SetSize(size+size/10 + 8);			// Guess that 10% extra is enough
	for (int i=0;i<size;i++)
	{
		if (ptr[i]=='<')						// < -> &lt;
			text.Add(TEXT("&lt;"));
		else if (ptr[i]=='&')					// & -> &amp;
			text.Add(TEXT("&amp;"));
		else if (ptr[i]=='>')					// > -> &gt;
			text.Add(TEXT("&gt;"));
		else if (ptr[i]=='\"')					// " -> &quot;
			text.Add(TEXT("&quot;"));
		else if (ptr[i]=='\'')					// ' -> &apos;
			text.Add(TEXT("&apos;"));
		else if (ptr[i]!='@' || ptr[i+1]!='@')	// @@ -> @
			text.Add(&ptr[i],1);
	}
}

void STRING::operator=(STRING &other)
{
	Set(other.ptr);
}

void STRING::operator=(TCHAR* other)
{
	Set(other);
}

BOOL STRING::operator==(TCHAR* other)
{
	BOOL result;

	if (ptr)
		result = !StrCmp(ptr,other);
	else
		result = !StrCmp(TEXT(""),other);
	return (result);
}
BOOL STRING::operator!=(TCHAR* other)
{
	BOOL result;
	if (ptr)
		result = StrCmp(ptr,other);
	else
		result = StrCmp(TEXT(""),other);
	return (result);
}

void IDTABLE::Increase()
{
	maxsize += 100;
	TCHAR* newtable = (TCHAR*) VirtualAlloc(NULL,(maxsize)*(XREFSIZE+1)*sizeof(TCHAR),MEM_COMMIT,PAGE_READWRITE);
	if (table)
	{
		memcpy(newtable,table,size*(XREFSIZE+1)*sizeof(TCHAR));
		VirtualFree(table,0,MEM_RELEASE);
	}
	table = newtable;
}

IDTABLE::IDTABLE()
{
	size = 0;
	maxsize = 0;
	table = NULL;
}

IDTABLE::~IDTABLE()
{
	if (table)
		VirtualFree(table,0,MEM_RELEASE);
}

long IDTABLE::Add(TCHAR* str)
{
	if (size>=maxsize)
		Increase();
	StringCchCopy(&table[size*(XREFSIZE+1)],XREFSIZE+1,str);
	size++;
	return size;
}

long IDTABLE::Find(TCHAR* str)
{
	for (long i=0;i<size;i++)
	{
		if (!StrCmp(&table[i*(XREFSIZE+1)],str))
			return i+1;
	}
	return 0;
}

void IDLIST::Increase()
{
	maxsize += 10;
	long* newlist = (long*) VirtualAlloc(NULL,maxsize*sizeof(long),MEM_COMMIT,PAGE_READWRITE);
	if (list)
	{
		memcpy(newlist,list,size*sizeof(long));
		VirtualFree(list,0,MEM_RELEASE);
	}
	list = newlist;
}

IDLIST::IDLIST()
{
	size = 0;
	maxsize = 0;
	list = NULL;
}

IDLIST::~IDLIST()
{
	if (list)
		VirtualFree(list,0,MEM_RELEASE);
}

long IDLIST::Add(long id)
{
	if (size>=maxsize)
		Increase();
	list[size] = id;
	size++;
	return size;
}
