/****************************************************************
 *                                                              *
 *  Copyright (C) 2003,2005 Christoffer Owe                     *
 *  For conditions of distribution and use, see License.txt     *
 *                                                              *
 ****************************************************************/

#ifndef _COMMON
#define _COMMON

#include <windows.h>
#include <crtdbg.h>

class STRING
{
private:
	TCHAR*	ptr;
	long	size;
	long	MaxSize;

	void	SetSize(long length);

public:
	STRING();
	STRING(TCHAR* text);
	~STRING();
	long Size();
	void Add(STRING &string);
	void Add(const TCHAR* string);
	void Add(const TCHAR* string,int length);
	void Set(const TCHAR* string);
	void Set(const TCHAR* string,int length);
	int Get(TCHAR* string,int length);
	TCHAR* Get();
	void ConvertToXML(STRING&);
	void operator=(STRING &other);
	void operator=(TCHAR* other);
	BOOL STRING::operator==(TCHAR* other);
	BOOL STRING::operator!=(TCHAR* other);
};

#define XREFSIZE	20

class IDTABLE
{
private:
	TCHAR *table;
	long size;
	long maxsize;

	void Increase();
public:
	IDTABLE();
	~IDTABLE();
	long Add(TCHAR*);
	long Find(TCHAR*);
	inline long GetSize(){return size;}
	inline TCHAR* Get(int i){return &table[(i-1)*(XREFSIZE+1)];}
};

class IDLIST
{
private:
	long *list;
	long size;
	long maxsize;

	void Increase();
public:
	IDLIST();
	~IDLIST();
	long Add(long id);
	inline long GetSize(){return size;}
	inline long Get(int i){return list[i-1];}
};

template<class T>
class VECTOR
{
private:
	T* vector;
	long size;
	long maxsize;

	void Increase()
	{
		maxsize += 100;
		T* newvector = (T*) VirtualAlloc(NULL,(maxsize)*sizeof(T),MEM_COMMIT,PAGE_READWRITE);
		if (!newvector)
		{
			DWORD error = GetLastError();
		}
		_ASSERT(!IsBadWritePtr(newvector,maxsize*sizeof(T)));
		if (vector)
		{
			for (int i=0;i<size;i++)
				newvector[i] = vector[i];
			VirtualFree(vector,0,MEM_RELEASE);
		}
		vector = newvector;
	}
public:
	VECTOR()
	{
		size = 0;
		maxsize = 0;
		vector = NULL; //(T*) VirtualAlloc(NULL,(maxsize)*sizeof(T),MEM_COMMIT,PAGE_READWRITE);
	}
	~VECTOR()
	{
		if (vector)
		{
			for (long i=0;i<size;i++)
				delete vector[i];
			VirtualFree(vector,0,MEM_RELEASE);
		}
	}
	long Add(T obj)
	{
		if (size>=maxsize)
			Increase();
		vector[size] = obj;
		size++;
		return size;
	}
	T Get(long index)
	{
		return vector[index-1];
	}
	long GetSize()
	{
		return size;
	}
};

#endif