/****************************************************************
 *                                                              *
 *  Copyright (C) 2003-2009 Christoffer Owe                     *
 *  For conditions of distribution and use, see License.txt     *
 *                                                              *
 ****************************************************************/

#include <windows.h>
#include <Shlwapi.h>
#include <strsafe.h>
//#include <stdio.h>
#include "genxml.h"
#include "ged2xml.h"

GENXML::GENXML(HANDLE file,CONVERSIONDATA* convdata)
{
	static WCHAR ByteOrderMark[2] = {0xFEFF,0};
	static TCHAR XMLHeaderStart[]=TEXT("<?xml version=\"1.0\" encoding=\"");
	static TCHAR XMLHeaderEnd[]=TEXT("\"?>");
	static TCHAR CharAnsi[]=TEXT("ISO-8859-1");
	static TCHAR CharUtf8[]=TEXT("UTF-8");
	static TCHAR CharUnicode[]=TEXT("UTF-16");
	static TCHAR Start[]=TEXT("<genxml xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:noNamespaceSchemaLocation=\"genxml20.xsd\">");
	static TCHAR File[]=TEXT("<file><version>200</version><level>3</level></file>");

	ConvData = convdata;
	GXFile = file;
	// Write byte order mark
	if (ConvData->CharacterSet==2) // Unicode
		Write(ByteOrderMark);
	// Write XML header
	Write(XMLHeaderStart);
	if (ConvData->CharacterSet==0) // ANSI
		Write(CharAnsi);
	else if (ConvData->CharacterSet==1) // UTF-8
		Write(CharUtf8);
	else
		Write(CharUnicode);
	Write(XMLHeaderEnd);
	// GenXML header
	Write(Start);
	Write(File);
}

GENXML::~GENXML()
{
	static TCHAR End[]=TEXT("</genxml>");

	Write(End);
}

void GENXML::Write(TCHAR* szText)
{
	DWORD BytesWritten;

	if (ConvData->CharacterSet==0) // ANSI
	{
		int cbSize = lstrlen(szText) + 1;
		char* szConv = (char*) HeapAlloc(GetProcessHeap(),HEAP_ZERO_MEMORY,cbSize);
		int cch = WideCharToMultiByte(CP_ACP,0,szText,-1,szConv,cbSize,NULL,NULL);
		WriteFile(GXFile,szConv,cch-1,&BytesWritten,NULL);
		HeapFree(GetProcessHeap(),0,szConv);
	}
	else if (ConvData->CharacterSet==1) // UTF-8
	{
		int cbSize = lstrlen(szText)*sizeof(TCHAR) + 8;
		char* szConv = (char*) HeapAlloc(GetProcessHeap(),HEAP_ZERO_MEMORY,cbSize);
		int cch = WideCharToMultiByte(CP_UTF8,0,szText,-1,szConv,cbSize,NULL,NULL);
		WriteFile(GXFile,szConv,cch-1,&BytesWritten,NULL);
		HeapFree(GetProcessHeap(),0,szConv);
	}
	else // Unicode
		WriteFile(GXFile,szText,lstrlen(szText)*sizeof(TCHAR),&BytesWritten,NULL);
}

void GENXML::Write(STRING &string)
{
	//DWORD	BytesWritten;
	STRING	conv;

	string.ConvertToXML(conv);
	Write(conv.Get());
	//WriteFile(GXFile,conv.Get(),conv.Size(),&BytesWritten,NULL);
}

void GENXML::Header(GC_HEADER &header,VECTOR<GC_SUBMITTER*>& Submitters)
{
	static TCHAR ExpSys[]=TEXT("<header><exportingsystem>");
	static TCHAR Version[]=TEXT("</exportingsystem><version>");
	static TCHAR VersionEnd[]=TEXT("</version>");
	static TCHAR End[]=TEXT("</header>");
	static TCHAR szEmailStart[]=TEXT("<email>");
	static TCHAR szEmailEnd[]=TEXT("</email>");
	static TCHAR szOwnerStart[]=TEXT("<owner>");
	static TCHAR szOwnerEnd[]=TEXT("</owner>");
	static TCHAR szNameStart[]=TEXT("<name>");
	static TCHAR szNameEnd[]=TEXT("</name>");

	Write(ExpSys);
	if (header.SourName!=TEXT(""))
		Write(header.SourName);
	else if (header.Sour!=TEXT(""))
		Write(header.Sour);
	else
		Write(TEXT("Unknown"));
	Write(Version);
	if (header.SourVers!=TEXT(""))
		Write(header.SourVers);
	else
		Write(TEXT("Unknown"));
	Write(VersionEnd);
	SimpleDate(header.Date,TEXT("exported"),NULL,TRUE);
	if (header.Submitter[0])
	{
		for (int i=1;i<=Submitters.GetSize();i++)
		{
			if (!StrCmp(header.Submitter,Submitters.Get(i)->id))
			{
				Write(szOwnerStart);
				Write(szNameStart);
				Write(Submitters.Get(i)->Name);
				Write(szNameEnd);
				Address(Submitters.Get(i)->Address);
				Write(szEmailStart);
				Write(Submitters.Get(i)->Email);
				Write(szEmailEnd);
				Write(szOwnerEnd);
				break;
			}
		}
	}
	Write(End);
}

long GENXML::Repository(VECTOR<GC_REPOSITORY*> &Repositories,VECTOR<GC_NOTEREC*> &Notes,IDTABLE &NoteIDs)
{
	static TCHAR szRepStart1[]=TEXT("<repository id=\"");
	static TCHAR szRepStart2[]=TEXT("\">");
	static TCHAR szRepEnd[]=TEXT("</repository>");
	static TCHAR szNameStart[]=TEXT("<name>");
	static TCHAR szNameEnd[]=TEXT("</name>");
	static TCHAR szEmailStart[]=TEXT("<email>");
	static TCHAR szEmailEnd[]=TEXT("</email>");
	static TCHAR szURIStart[]=TEXT("<uri>");
	static TCHAR szURIEnd[]=TEXT("</uri>");
	TCHAR tmp[32];
	GC_REPOSITORY* Repository;

	for (long i=1;i<=Repositories.GetSize();i++)
	{
		Repository = Repositories.Get(i);
		Write(szRepStart1);
		StringCbPrintf(tmp,sizeof(tmp),TEXT("R%i"),i);
		Write(tmp);
		Write(szRepStart2);
		// Write name
		Write(szNameStart);
		Write(Repository->Name);
		Write(szNameEnd);
		// Write address
		Address(Repository->Address);
		// Write email
		if (Repository->Address.Email!=TEXT(""))
		{
			Write(szEmailStart);
			Write(Repository->Address.Email);
			Write(szEmailEnd);
		}
		// Write URI
		if (Repository->Address.WWW!=TEXT(""))
		{
			Write(szURIStart);
			Write(Repository->Address.WWW);
			Write(szURIEnd);
		}
		// Write note
		Note(Repository->Note,Notes,NoteIDs);
		// Write change
		Change(Repository->Change);
		Write(szRepEnd);
	}
	return Repositories.GetSize();
}

long GENXML::Source(GEDCOMDATA* data,int ObjectStatus)
{
	static TCHAR szSourceStart1[]=TEXT("<source id=\"");
	static TCHAR szSourceStart2[]=TEXT("\">");
	static TCHAR szSourceEnd[]=TEXT("</source>");
	static TCHAR szAuthorStart[]=TEXT("<author>");
	static TCHAR szAuthorEnd[]=TEXT("</author>");
	static TCHAR szTitleStart[]=TEXT("<title>");
	static TCHAR szTitleEnd[]=TEXT("</title>");
	static TCHAR szShortStart[]=TEXT("<shorttitle>");
	static TCHAR szShortEnd[]=TEXT("</shorttitle>");
	static TCHAR szPublishedStart[]=TEXT("<published>");
	static TCHAR szPublishedEnd[]=TEXT("</published>");
	static TCHAR szReprefStart[]=TEXT("<repositoryref ref=\"");
	static TCHAR szReprefEnd[]=TEXT("</repositoryref>");
	static TCHAR szCallStart[]=TEXT("<callnumber>");
	static TCHAR szCallEnd[]=TEXT("</callnumber>");
	static TCHAR szTextStart[]=TEXT("<text>");
	static TCHAR szTextEnd[]=TEXT("</text>");
	TCHAR tmp[32];
	long id,nSources;
	GC_SOURCE* Source;
	GC_SOURCECITE* sc;

	for (long i=1;i<=data->Sources.GetSize();i++)
	{
		Source = data->Sources.Get(i);
		Write(szSourceStart1);
		StringCbPrintf(tmp,sizeof(tmp),TEXT("S%i"),i);
		Write(tmp);
		Write(szSourceStart2);
		// Write author
		Write(szAuthorStart);
		Write(Source->Author.Text);
		Write(szAuthorEnd);
		// Write title
		Write(szTitleStart);
		Write(Source->Title.Text);
		Write(szTitleEnd);
		// Write short title
		Write(szShortStart);
		Write(Source->Abbreviation);
		Write(szShortEnd);
		// Write published
		if (Source->Publication.Text!=TEXT(""))
		{
			Write(szPublishedStart);
			Write(Source->Publication.Text);
			Write(szPublishedEnd);
		}
		// Repositoryref
		if (Source->Repref.Rep[0] && (id=data->RepositoryIDs.Find(Source->Repref.Rep)>0))
		{
			Write(szReprefStart);
			StringCbPrintf(tmp,sizeof(tmp),TEXT("R%i"),id);
			Write(tmp);
			Write(szSourceStart2);
			if (Source->Repref.Callnumber!=TEXT(""))
			{
				Write(szCallStart);
				Write(Source->Repref.Callnumber);
				Write(szCallEnd);
			}
			Note(Source->Repref.Note,data->Notes,data->NoteIDs);
			Write(szReprefEnd);
		}
		// Write text
		if (Source->Text.Text!=TEXT(""))
		{
			Write(szTextStart);
			Write(Source->Text.Text);
			Write(szTextEnd);
		}
		// Write note
		Note(Source->Note,data->Notes,data->NoteIDs);
		// Write objects
		for (int j=1;j<=Source->Objects.GetSize();j++)
			Object(*Source->Objects.Get(j),data->Objects,data->ObjectIDs,data->Notes,data->NoteIDs,ObjectStatus);
		// Write change
		Change(Source->Change);
		Write(szSourceEnd);
	}
	nSources = data->Sources.GetSize();
	// Generate sources from unstructured source cites
	for (i=1;i<=data->Sourcerefs.GetSize();i++)
	{
		sc = data->Sourcerefs.Get(i);
		if (!sc->SourceID[0])
		{
			nSources++;
			Write(szSourceStart1);
			StringCbPrintf(tmp,sizeof(tmp),TEXT("S%i"),nSources);
			Write(tmp);
			Write(szSourceStart2);
			sc->NewSourceID = nSources;	// Store the id for later reference
			// Write author
			Write(szAuthorStart);
			Write(TEXT(""));
			Write(szAuthorEnd);
			// Write title
			Write(szTitleStart);
			Write(sc->SourceDescription.Text);
			Write(szTitleEnd);
			// Write short title
			Write(szShortStart);
			Write(TEXT(""));
			Write(szShortEnd);
			// Write note
			Note(sc->Note,data->Notes,data->NoteIDs);
			Write(szSourceEnd);
		}
	}
	return nSources;
}

long GENXML::Excerpt(VECTOR<GC_SOURCECITE*> &Sourcerefs,IDTABLE& SourceIDs,VECTOR<GC_NOTEREC*> &Notes,IDTABLE &NoteIDs)
{
	static TCHAR szExcerptStart1[]=TEXT("<excerpt id=\"");
	static TCHAR szExcerptStart2[]=TEXT("\">");
	static TCHAR szExcerptEnd[]=TEXT("</excerpt>");
	static TCHAR szQualityStart[]=TEXT("<quality>");
	static TCHAR szQualityEnd[]=TEXT("</quality>");
	static TCHAR szPageStart[]=TEXT("<page>");
	static TCHAR szPageEnd[]=TEXT("</page>");
	static TCHAR szTextStart[]=TEXT("<text>");
	static TCHAR szTextEnd[]=TEXT("</text>");
	static TCHAR szRefStart[]=TEXT("<sourceref>");
	static TCHAR szRefEnd[]=TEXT("</sourceref>");
	TCHAR tmp[32];
	TCHAR* ptr;
	long id;
	GC_SOURCECITE* sc;

	for (long i=1;i<=Sourcerefs.GetSize();i++)
	{
		sc = Sourcerefs.Get(i);
		Write(szExcerptStart1);
		StringCbPrintf(tmp,sizeof(tmp),TEXT("X%i"),i);
		Write(tmp);
		Write(szExcerptStart2);
		// Write text
		if (sc->TextDate!=TEXT("") || sc->Text.Text!=TEXT(""))
		{
			Write(szTextStart);
			if (sc->TextDate!=TEXT(""))
			{
				ptr = sc->TextDate.Get();
				if (ptr[0]=='(') ptr++;
				StringCbCopy(tmp,sizeof(tmp),ptr);
				if (tmp[lstrlen(tmp)-1]==')')
					tmp[lstrlen(tmp)-1] = 0;
				StringCbCat(tmp,sizeof(tmp),TEXT(":\r\n"));
				Write(tmp);
			}
			if (sc->Text.Text!=TEXT(""))
				Write(sc->Text.Text);
			Write(szTextEnd);
		}
		if (sc->SourceID[0])	// structured source cite
		{
			// Write quality
			if (sc->Quality!=TEXT(""))
			{
				Write(szQualityStart);
				if (sc->Quality==TEXT("3"))
					Write(TEXT("98"));
				else if (sc->Quality==TEXT("2"))
					Write(TEXT("95"));
				else if (sc->Quality==TEXT("1"))
					Write(TEXT("90"));
				else
					Write(TEXT("80"));
				Write(szQualityEnd);
			}
			// Write page
			if (sc->Page!=TEXT(""))
			{
				Write(szPageStart);
				Write(sc->Page);
				Write(szPageEnd);
			}
			// Write sourceref
			id = SourceIDs.Find(sc->SourceID);
		}
		else					// unstructured source cite
		{
			// Write page (Brothers Keepers)
			if (sc->Page!=TEXT(""))
			{
				Write(szPageStart);
				Write(sc->Page);
				Write(szPageEnd);
			}
			// Find source ID
			id = sc->NewSourceID;
		}
		_ASSERT(id);
		Write(szRefStart);
		StringCbPrintf(tmp,sizeof(tmp),TEXT("S%i"),id);
		Write(tmp);
		Write(szRefEnd);
		// Write note
		Note(sc->Note,Notes,NoteIDs);
		Write(szExcerptEnd);
	}
	return Sourcerefs.GetSize();
}

long GENXML::Eventtype(VECTOR<EVENTTYPE*> &Eventtypes)
{
	static TCHAR szEventtypeStart1[]=TEXT("<eventtype id=\"");
	static TCHAR szEventtypeStart2[]=TEXT("\" class=\"");
	static TCHAR szEventtypeStart3[]=TEXT("\">");
	static TCHAR szEventtypeEnd[]=TEXT("</eventtype>");
	static TCHAR szDescrStart[]=TEXT("<description>");
	static TCHAR szDescrEnd[]=TEXT("</description>");
	static TCHAR szTagStart[]=TEXT("<gedcomtag>");
	static TCHAR szTagEnd[]=TEXT("</gedcomtag>");
	static TCHAR szRolesStart[]=TEXT("<roles>");
	static TCHAR szRolesEnd[]=TEXT("</roles>");
	TCHAR tmp[32];
	TCHAR* descr;
	EVENTTYPE* et;

	for (long i=1;i<=Eventtypes.GetSize();i++)
	{
		et = Eventtypes.Get(i);
		Write(szEventtypeStart1);
		StringCbPrintf(tmp,sizeof(tmp),TEXT("E%i"),i);
		Write(tmp);
		Write(szEventtypeStart2);
		switch (et->tag)
		{
		case TAG_BIRT:
			Write(TEXT("birth"));
			descr = ConvData->MsgProvider(EVENTTYPE_BIRT);
			break;
		case TAG_CHR:
			Write(TEXT("baptism"));
			descr = ConvData->MsgProvider(EVENTTYPE_CHR);
			break;
		case TAG_DEAT:
			Write(TEXT("death"));
			descr = ConvData->MsgProvider(EVENTTYPE_DEAT);
			break;
		case TAG_BURI:
			Write(TEXT("burial"));
			descr = ConvData->MsgProvider(EVENTTYPE_BURI);
			break;
		case TAG_CREM:
			Write(TEXT("cremation"));
			descr = ConvData->MsgProvider(EVENTTYPE_CREM);
			break;
		case TAG_ADOP:
			Write(TEXT("adoption"));
			descr = ConvData->MsgProvider(EVENTTYPE_ADOP);
			break;
		case TAG_BAPM:
			Write(TEXT("baptism"));
			descr = ConvData->MsgProvider(EVENTTYPE_BAPM);
			break;
		case TAG_BARM:
			Write(TEXT("confirmation"));
			descr = ConvData->MsgProvider(EVENTTYPE_BARM);
			break;
		case TAG_BASM:
			Write(TEXT("confirmation"));
			descr = ConvData->MsgProvider(EVENTTYPE_BASM);
			break;
		case TAG_BLES:
			Write(TEXT("blessing"));
			descr = ConvData->MsgProvider(EVENTTYPE_BLES);
			break;
		case TAG_CHRA:
			Write(TEXT("baptism"));
			descr = ConvData->MsgProvider(EVENTTYPE_CHRA);
			break;
		case TAG_CONF:
			Write(TEXT("confirmation"));
			descr = ConvData->MsgProvider(EVENTTYPE_CONF);
			break;
		case TAG_FCOM:
			Write(TEXT("other"));
			descr = ConvData->MsgProvider(EVENTTYPE_FCOM);
			break;
		case TAG_ORDN:
			Write(TEXT("ordination"));
			descr = ConvData->MsgProvider(EVENTTYPE_ORDN);
			break;
		case TAG_NATU:
			Write(TEXT("naturalization"));
			descr = ConvData->MsgProvider(EVENTTYPE_NATU);
			break;
		case TAG_EMIG:
			Write(TEXT("emigration"));
			descr = ConvData->MsgProvider(EVENTTYPE_EMIG);
			break;
		case TAG_IMMI:
			Write(TEXT("immigration"));
			descr = ConvData->MsgProvider(EVENTTYPE_IMMI);
			break;
		case TAG_CENS:
			Write(TEXT("census"));
			descr = ConvData->MsgProvider(EVENTTYPE_CENS);
			break;
		case TAG_PROB:
			Write(TEXT("other"));
			descr = ConvData->MsgProvider(EVENTTYPE_PROB);
			break;
		case TAG_WILL:
			Write(TEXT("other"));
			descr = ConvData->MsgProvider(EVENTTYPE_WILL);
			break;
		case TAG_GRAD:
			Write(TEXT("graduation"));
			descr = ConvData->MsgProvider(EVENTTYPE_GRAD);
			break;
		case TAG_RESI:
			Write(TEXT("residence"));
			descr = ConvData->MsgProvider(EVENTTYPE_RESI);
			break;
		case TAG_RETI:
			Write(TEXT("retirement"));
			descr = ConvData->MsgProvider(EVENTTYPE_RETI);
			break;
		case TAG_ANUL:
			Write(TEXT("annulment"));
			descr = ConvData->MsgProvider(EVENTTYPE_ANUL);
			break;
		case TAG_DIV:
			Write(TEXT("divorce"));
			descr = ConvData->MsgProvider(EVENTTYPE_DIV);
			break;
		case TAG_DIVF:
			Write(TEXT("divorce"));
			descr = ConvData->MsgProvider(EVENTTYPE_DIVF);
			break;
		case TAG_ENGA:
			Write(TEXT("marriage"));
			descr = ConvData->MsgProvider(EVENTTYPE_ENGA);
			break;
		case TAG_MARR:
			Write(TEXT("marriage"));
			descr = ConvData->MsgProvider(EVENTTYPE_MARR);
			break;
		case TAG_MARB:
			Write(TEXT("marriage"));
			descr = ConvData->MsgProvider(EVENTTYPE_MARB);
			break;
		case TAG_MARC:
			Write(TEXT("marriage"));
			descr = ConvData->MsgProvider(EVENTTYPE_MARC);
			break;
		case TAG_MARL:
			Write(TEXT("marriage"));
			descr = ConvData->MsgProvider(EVENTTYPE_MARL);
			break;
		case TAG_MARS:
			Write(TEXT("marriage"));
			descr = ConvData->MsgProvider(EVENTTYPE_MARS);
			break;
		case TAG_BAPL:
			Write(TEXT("baptism"));
			descr = ConvData->MsgProvider(EVENTTYPE_BAPL);
			break;
		case TAG_CONL:
			Write(TEXT("confirmation"));
			descr = ConvData->MsgProvider(EVENTTYPE_CONL);
			break;
		case TAG_ENDL:
			Write(TEXT("other"));
			descr = ConvData->MsgProvider(EVENTTYPE_ENDL);
			break;
		case TAG_SLGC:
			Write(TEXT("other"));
			descr = ConvData->MsgProvider(EVENTTYPE_SLGC);
			break;
		case TAG_SLGS:
			Write(TEXT("marriage"));
			descr = ConvData->MsgProvider(EVENTTYPE_SLGS);
			break;
		default:
			Write(TEXT("other"));
			descr = TEXT("");
			break;
		}
		Write(szEventtypeStart3);
		Write(szDescrStart);
		if (et->description==TEXT(""))
			Write(descr);
		else
			Write(et->description);
		Write(szDescrEnd);
		Write(szTagStart);
		StringCchCopy(tmp,5,&GedcomTags[(et->tag-1)*5]);
		tmp[5] = 0;
		if (tmp[4]==' ')
			tmp[4] = 0;
		if (tmp[3]==' ')
			tmp[3] = 0;
		Write(tmp);
		Write(szTagEnd);
		Write(szRolesStart);
		Write(_itow(et->roles,tmp,10));
		Write(szRolesEnd);
		Write(szEventtypeEnd);
	}
	return Eventtypes.GetSize();
}

long GENXML::Person(GEDCOMDATA* data,int ObjectStatus)
{
	static TCHAR szPersonStart1[]=TEXT("<person id=\"");
	static TCHAR szPersonStart2[]=TEXT("\" sex=\"");
	static TCHAR szPersonStart3[]=TEXT("\">");
	static TCHAR szPersonEnd[]=TEXT("</person>");
	static TCHAR szUnknown[]=TEXT("<personalname><np tp=\"unkw\">Unknown</np></personalname>");
	static TCHAR szNoteStart[]=TEXT("<note>");
	static TCHAR szNoteEnd[]=TEXT("</note>");
	TCHAR tmp[32];
	GC_PERSON* Person;

	for (long i=1;i<=data->Persons.GetSize();i++)
	{
		Person = data->Persons.Get(i);
		Write(szPersonStart1);
		StringCbPrintf(tmp,sizeof(tmp),TEXT("P%i"),Person->NewID);
		Write(tmp);
		Write(szPersonStart2);
		if (Person->Sex==1)
			Write(TEXT("male"));
		else if (Person->Sex==2)
			Write(TEXT("female"));
		else
			Write(TEXT("unknown"));
		Write(szPersonStart3);
		// Write name
		if (Person->Names.GetSize())
			PersonalName(*Person->Names.Get(1));
		else
			Write(szUnknown);
		// Write excerptref
		Excerptref(Person->SourcerefIDs);
		// Write objects
		for (int j=1;j<=Person->Objects.GetSize();j++)
			Object(*Person->Objects.Get(j),data->Objects,data->ObjectIDs,data->Notes,data->NoteIDs,ObjectStatus);
		// Write comments
		if (Person->Comments.Text!=TEXT(""))
		{
			Write(szNoteStart);
			Write(Person->Comments.Text);
			Write(szNoteEnd);
		}
		// Write change
		Change(Person->Change);
		Write(szPersonEnd);
	}
	return data->Persons.GetSize();
}

long GENXML::Assertion(VECTOR<GC_PERSON*> &Persons,VECTOR<GC_FAMILY*> &Families,IDTABLE &ids,VECTOR<GC_NOTEREC*> &Notes,IDTABLE &NoteIDs)
{
	static TCHAR szAssertionStart1[]=TEXT("<assertion id=\"");
	static TCHAR szAssertionStart2[]=TEXT("\">");
	static TCHAR szAssertionEnd[]=TEXT("</assertion>");
	static TCHAR szAliasStart[]=TEXT("<alias>");
	static TCHAR szAliasEnd[]=TEXT("</alias>");
	static TCHAR szEventStart1[]=TEXT("<event type=\"");
	static TCHAR szEventStart2[]=TEXT("\">");
	static TCHAR szEventEnd[]=TEXT("</event>");
	static TCHAR szPrincipalStart[]=TEXT("<principal>");
	static TCHAR szPrincipalEnd[]=TEXT("</principal>");
	static TCHAR szAttributeStart[]=TEXT("<attribute>");
	static TCHAR szAttributeEnd[]=TEXT("</attribute>");
	static TCHAR szPersonrefStart[]=TEXT("<personref>");
	static TCHAR szPersonrefEnd[]=TEXT("</personref>");
	static TCHAR szDescrStart[]=TEXT("<description>");
	static TCHAR szDescrEnd[]=TEXT("</description>");
	static TCHAR szTextclassStart[]=TEXT("<textclass>");
	static TCHAR szTextclassEnd[]=TEXT("</textclass>");
	static TCHAR szTextStart[]=TEXT("<text>");
	static TCHAR szTextEnd[]=TEXT("</text>");
	static TCHAR szInfoStart[]=TEXT("<info>");
	static TCHAR szInfoEnd[]=TEXT("</info>");
	static TCHAR szRelationshipStart[]=TEXT("<relationship>");
	static TCHAR szRelationshipEnd[]=TEXT("</relationship>");
	static TCHAR szRelation[]=TEXT("<relation>biological</relation>");
	static TCHAR szFatherStart[]=TEXT("<father>");
	static TCHAR szFatherEnd[]=TEXT("</father>");
	static TCHAR szMotherStart[]=TEXT("<mother>");
	static TCHAR szMotherEnd[]=TEXT("</mother>");
	static TCHAR szChildStart[]=TEXT("<child>");
	static TCHAR szChildEnd[]=TEXT("</child>");
	TCHAR tmp[32];
	int descr;
	TCHAR* textclass;
	GC_PERSON* Person;
	GC_FAMILY* Family;
	GC_NAME* Name;
	GC_EVENT* Event;
	GC_ATTRIBUTE* Attribute;
	GC_NOTE* NoteStr;
	long i,k,iAssertions;
	long father,mother,child;

	iAssertions = 0;
	// Write Person assertions
	for (i=1;i<=Persons.GetSize();i++)
	{
		Person = Persons.Get(i);
		// Write alias
		for (k=1;k<=Person->Names.GetSize();k++)
		{
			Name = Person->Names.Get(k);
			if (k>1 || Name->SourcerefIDs.GetSize() || Name->Note.Note!=TEXT("") || Name->Note.NoteID[0])
			{
				iAssertions++;
				Write(szAssertionStart1);
				StringCbPrintf(tmp,sizeof(tmp),TEXT("A%i"),iAssertions);
				Write(tmp);
				Write(szAssertionStart2);
				Write(szAliasStart);
				PersonalName(*Name);
				Write(szPersonrefStart);
				StringCbPrintf(tmp,sizeof(tmp),TEXT("P%i"),i);
				Write(tmp);
				Write(szPersonrefEnd);
				Write(szAliasEnd);
				Excerptref(Name->SourcerefIDs);
				Note(Name->Note,Notes,NoteIDs);
				Write(szAssertionEnd);
			}
		}
		// Write attributes
		for (k=1;k<=Person->Attributes.GetSize();k++)
		{
			iAssertions++;
			Attribute = Person->Attributes.Get(k);
			switch (Attribute->Tag)
			{
			case TAG_CAST:
				descr = EVENTTYPE_CAST;
				textclass = TEXT("caste");
				break;
			case TAG_DSCR:
				descr = EVENTTYPE_DSCR;
				textclass = TEXT("physical");
				break;
			case TAG_EDUC:
				descr = EVENTTYPE_EDUC;
				textclass = TEXT("education");
				break;
			case TAG_IDNO:
				descr = EVENTTYPE_IDNO;
				textclass = TEXT("idnumber");
				break;
			case TAG_NATI:
				descr = EVENTTYPE_NATI;
				textclass = TEXT("nationality");
				break;
			case TAG_NCHI:
				descr = EVENTTYPE_NCHI;
				textclass = TEXT("other");
				break;
			case TAG_NMR:
				descr = EVENTTYPE_NMR;
				textclass = TEXT("other");
				break;
			case TAG_OCCU:
				descr = EVENTTYPE_OCCU;
				textclass = TEXT("work");
				break;
			case TAG_PROP:
				descr = EVENTTYPE_PROP;
				textclass = TEXT("property");
				break;
			case TAG_RELI:
				descr = EVENTTYPE_RELI;
				textclass = TEXT("religion");
				break;
			case TAG_SSN:
				descr = EVENTTYPE_SSN;
				textclass = TEXT("idnumber");
				break;
			case TAG_TITL:
				descr = EVENTTYPE_TITL;
				textclass = TEXT("title");
				break;
			case UTAG_UID:
				descr = EVENTTYPE_UID;
				textclass = TEXT("other");
				break;
			case UTAG_EMAIL:
				descr = EVENTTYPE_EMAIL;
				textclass = TEXT("email");
				break;
			default:	// Including FACT
				descr = 0;
				textclass = TEXT("other");
				break;
			}
			Write(szAssertionStart1);
			StringCbPrintf(tmp,sizeof(tmp),TEXT("A%i"),iAssertions);
			Write(tmp);
			Write(szAssertionStart2);
			Write(szAttributeStart);
			Write(szDescrStart);
			if (Attribute->Type!=TEXT(""))
				Write(Attribute->Type);
			else
				Write(ConvData->MsgProvider(descr));
			Write(szDescrEnd);
			Write(szTextclassStart);
			Write(textclass);
			Write(szTextclassEnd);
			Write(szTextStart);
			Write(Attribute->Text);
			Write(szTextEnd);
			Write(szPersonrefStart);
			StringCbPrintf(tmp,sizeof(tmp),TEXT("P%i"),i);
			Write(tmp);
			Write(szPersonrefEnd);
			Date(Attribute->Date);
			Place(Attribute->Place);
			Address(Attribute->Address);
			Write(szAttributeEnd);
			Excerptref(Attribute->SourcerefIDs);
			Note(Attribute->Note,Notes,NoteIDs);
			Write(szAssertionEnd);
		}
		// Write events
		for (k=1;k<=Person->Events.GetSize();k++)
		{
			iAssertions++;
			Event = Person->Events.Get(k);
			Write(szAssertionStart1);
			StringCbPrintf(tmp,sizeof(tmp),TEXT("A%i"),iAssertions);
			Write(tmp);
			Write(szAssertionStart2);
			Write(szEventStart1);
			StringCbPrintf(tmp,sizeof(tmp),TEXT("E%i"),Event->EventType);
			Write(tmp);
			Write(szEventStart2);
			Write(szPrincipalStart);
			StringCbPrintf(tmp,sizeof(tmp),TEXT("P%i"),i);
			Write(tmp);
			Write(szPrincipalEnd);
			Date(Event->Date);
			Place(Event->Place);
			Address(Event->Address);
			Write(szEventEnd);
			Excerptref(Event->SourcerefIDs);
			Note(Event->Note,Notes,NoteIDs);
			Write(szAssertionEnd);
		}
		// Write infos
		for (k=1;k<=Person->Notes.GetSize();k++)
		{
			iAssertions++;
			NoteStr = Person->Notes.Get(k);
			Write(szAssertionStart1);
			StringCbPrintf(tmp,sizeof(tmp),TEXT("A%i"),iAssertions);
			Write(tmp);
			Write(szAssertionStart2);
			Write(szInfoStart);
			Write(szTextStart);
			NoteText(NoteStr,Notes,NoteIDs);
			Write(szTextEnd);
			Write(szPersonrefStart);
			StringCbPrintf(tmp,sizeof(tmp),TEXT("P%i"),i);
			Write(tmp);
			Write(szPersonrefEnd);
			Write(szInfoEnd);
			Excerptref(NoteStr->SourcerefIDs);
			Write(szAssertionEnd);
		}
	}
	// Write family assertions
	for (i=1;i<=Families.GetSize();i++)
	{
		Family = Families.Get(i);
		if (Family->Husb[0])
			father = ids.Find(Family->Husb);
		else
			father = 0;
		if (Family->Wife[0])
			mother = ids.Find(Family->Wife);
		else
			mother = 0;
		if (!father && !mother)
			continue;
		// Write relationships
		for (k=1;k<=Family->Children.GetSize();k++)
		{
			iAssertions++;
			child = ids.Find(Family->Children.Get(k));
			_ASSERT(child);
			Write(szAssertionStart1);
			StringCbPrintf(tmp,sizeof(tmp),TEXT("A%i"),iAssertions);
			Write(tmp);
			Write(szAssertionStart2);
			Write(szRelationshipStart);
			Write(szRelation);
			if (father)
			{
				Write(szFatherStart);
				StringCbPrintf(tmp,sizeof(tmp),TEXT("P%i"),father);
				Write(tmp);
				Write(szFatherEnd);
			}
			if (mother)
			{
				Write(szMotherStart);
				StringCbPrintf(tmp,sizeof(tmp),TEXT("P%i"),mother);
				Write(tmp);
				Write(szMotherEnd);
			}
			Write(szChildStart);
			StringCbPrintf(tmp,sizeof(tmp),TEXT("P%i"),child);
			Write(tmp);
			Write(szChildEnd);
			Write(szRelationshipEnd);
			Excerptref(Family->SourcerefIDs);
			Write(szAssertionEnd);
		}
		// Write family events
		for (k=1;k<=Family->Events.GetSize();k++)
		{
			iAssertions++;
			Event = Family->Events.Get(k);
			Write(szAssertionStart1);
			StringCbPrintf(tmp,sizeof(tmp),TEXT("A%i"),iAssertions);
			Write(tmp);
			Write(szAssertionStart2);
			Write(szEventStart1);
			StringCbPrintf(tmp,sizeof(tmp),TEXT("E%i"),Event->EventType);
			Write(tmp);
			Write(szEventStart2);
			if (father)
			{
				Write(szPrincipalStart);
				StringCbPrintf(tmp,sizeof(tmp),TEXT("P%i"),father);
				Write(tmp);
				Write(szPrincipalEnd);
			}
			if (mother)
			{
				Write(szPrincipalStart);
				StringCbPrintf(tmp,sizeof(tmp),TEXT("P%i"),mother);
				Write(tmp);
				Write(szPrincipalEnd);
			}
			Date(Event->Date);
			Place(Event->Place);
			Address(Event->Address);
			Write(szEventEnd);
			Excerptref(Family->SourcerefIDs);
			Note(Event->Note,Notes,NoteIDs);
			Write(szAssertionEnd);
		}
		// Write family infos
		for (k=1;k<=Family->Notes.GetSize();k++)
		{
			iAssertions++;
			NoteStr = Family->Notes.Get(k);
			Write(szAssertionStart1);
			StringCbPrintf(tmp,sizeof(tmp),TEXT("A%i"),iAssertions);
			Write(tmp);
			Write(szAssertionStart2);
			Write(szInfoStart);
			Write(szTextStart);
			NoteText(NoteStr,Notes,NoteIDs);
			Write(szTextEnd);
			if (father)
			{
				Write(szPersonrefStart);
				StringCbPrintf(tmp,sizeof(tmp),TEXT("P%i"),father);
				Write(tmp);
				Write(szPersonrefEnd);
			}
			if (mother)
			{
				Write(szPersonrefStart);
				StringCbPrintf(tmp,sizeof(tmp),TEXT("P%i"),mother);
				Write(tmp);
				Write(szPersonrefEnd);
			}
			Write(szInfoEnd);
			Write(szAssertionEnd);
		}
	}
	return iAssertions;
}

long GENXML::Tasks(VECTOR<GC_TODO*>& Todos,IDTABLE& PersonIDs)
{
	//static TCHAR szObjective[]="<objective id=\"O1\"><title>Tasks</title><problem></problem><solution></solution></objective>";
	static TCHAR szTaskStart1[]=TEXT("<task id=\"");
	static TCHAR szTaskStart2[]=TEXT("\">");
	static TCHAR szTaskEnd[]=TEXT("</task>");
	static TCHAR szTitleStart[]=TEXT("<title>");
	static TCHAR szTitleEnd[]=TEXT("</title>");
	static TCHAR szProblemStart[]=TEXT("<problem>");
	static TCHAR szProblemEnd[]=TEXT("</problem>");
	static TCHAR szPersonrefStart[]=TEXT("<personref>");
	static TCHAR szPersonrefEnd[]=TEXT("</personref>");
	static TCHAR szStatus[]=TEXT("<status>new</status>");
	TCHAR tmp[32];
	GC_TODO* Todo;

	if (!Todos.GetSize())
		return 0;
	// Write a single objective
	//Write(szObjective);
	// Write the tasks
	for (long i=1;i<=Todos.GetSize();i++)
	{
		Todo = Todos.Get(i);
		Write(szTaskStart1);
		StringCbPrintf(tmp,sizeof(tmp),TEXT("T%i"),i);
		Write(tmp);
		Write(szTaskStart2);
		// Write title
		Write(szTitleStart);
		Write(Todo->Title);
		Write(szTitleEnd);
		// Write problem
		Write(szProblemStart);
		Write(Todo->Problem.Note);
		Write(szProblemEnd);
		// Write personref
		if (Todo->PersonID[0])
		{
			Write(szPersonrefStart);
			StringCbPrintf(tmp,sizeof(tmp),TEXT("P%i"),PersonIDs.Find(Todo->PersonID));
			Write(tmp);
			Write(szPersonrefEnd);
		}
		// Write status
		Write(szStatus);
		// Write date
		if (Todo->Date.DateType!=DATETYPE_NONE)
			SimpleDate(Todo->Date.Date1,TEXT("statusdate"),NULL,TRUE);
		Write(szTaskEnd);
	}
	return Todos.GetSize();
}

void GENXML::Total(long iRepositories,long iSources,long iExcerpts,long iEventtypes,long iPersons,long iAssertions,long iTasks)
{
	static TCHAR szRepositories[]=TEXT("<total><repositories>");
	static TCHAR szSources[]=TEXT("</repositories><sources>");
	static TCHAR szExcerpts[]=TEXT("</sources><excerpts>");
	static TCHAR szEventtypes[]=TEXT("</excerpts><eventtypes>");
	static TCHAR szPersons[]=TEXT("</eventtypes><persons>");
	static TCHAR szAssertions[]=TEXT("</persons><assertions>");
	static TCHAR szObjectives[]=TEXT("</assertions><objectives>");
	static TCHAR szTasks[]=TEXT("</objectives><tasks>");
	static TCHAR szEnd[]=TEXT("</tasks></total>");
	TCHAR tmp[16];

	Write(szRepositories);
	Write(_itow(iRepositories,tmp,10));
	Write(szSources);
	Write(_itow(iSources,tmp,10));
	Write(szExcerpts);
	Write(_itow(iExcerpts,tmp,10));
	Write(szEventtypes);
	Write(_itow(iEventtypes,tmp,10));
	Write(szPersons);
	Write(_itow(iPersons,tmp,10));
	Write(szAssertions);
	Write(_itow(iAssertions,tmp,10));
	Write(szObjectives);
	//if (iTasks)
	//	Write("1");
	//else
		Write(TEXT("0"));
	Write(szTasks);
	Write(_itow(iTasks,tmp,10));
	Write(szEnd);
}

void GENXML::Date(GC_DATE &date)
{
	TCHAR szDateStart[]=TEXT("<date>");
	TCHAR szDateEnd[]=TEXT("</date>");
	TCHAR szTextStart[]=TEXT("<text>");
	TCHAR szTextEnd[]=TEXT("</text>");

	if (date.DateType==DATETYPE_NONE)
		return;
	Write(szDateStart);
	if (date.DateType==DATETYPE_EXACT)
		SimpleDate(date.Date1,TEXT("exact"));
	else if (date.DateType==DATETYPE_BET)
	{
		SimpleDate(date.Date1,TEXT("begin"));
		SimpleDate(date.Date2,TEXT("end"));
	}
	else if (date.DateType==DATETYPE_FROMTO)
	{
		SimpleDate(date.Date1,TEXT("from"));
		SimpleDate(date.Date2,TEXT("to"));
	}
	else if (date.DateType==DATETYPE_FROM)
		SimpleDate(date.Date1,TEXT("from"));
	else if (date.DateType==DATETYPE_TO)
		SimpleDate(date.Date1,TEXT("to"));
	else if (date.DateType==DATETYPE_ABT)
		SimpleDate(date.Date1,TEXT("exact"),TEXT("about"));
	else if (date.DateType==DATETYPE_EST)
		SimpleDate(date.Date1,TEXT("exact"),TEXT("estimated"));
	else if (date.DateType==DATETYPE_AFT)
		SimpleDate(date.Date1,TEXT("exact"),TEXT("after"));
	else if (date.DateType==DATETYPE_BEF)
		SimpleDate(date.Date1,TEXT("exact"),TEXT("before"));
	else
	{
		_ASSERT(date.DateType==DATETYPE_TEXT);
		Write(szTextStart);
		Write(date.Text);
		Write(szTextEnd);
	}
	Write(szDateEnd);
}

void GENXML::SimpleDate(GC_SIMPLEDATE &date,TCHAR* tag,TCHAR* modifier,BOOL time)
{
	TCHAR buffer[32];
	TCHAR tmp[16];

	Write(TEXT("<"));
	Write(tag);
	if (date.calendar!=CALENDAR_UNKNOWN)
	{
		if (date.calendar==CALENDAR_HEBREW)
			Write(TEXT(" cal=\"hebrew\""));
		else if (date.calendar==CALENDAR_FRENCH)
			Write(TEXT(" cal=\"french\""));
		else if (date.calendar==CALENDAR_GREGORIAN)
			Write(TEXT(" cal=\"gregorian\""));
		else if (date.calendar==CALENDAR_JULIAN)
			Write(TEXT(" cal=\"julian\""));
		else
			Write(TEXT(" cal=\"unknown\""));
	}
	if (modifier)
	{
		Write(TEXT(" mod=\""));
		Write(modifier);
		Write(TEXT("\""));
	}
	if (date.year<0)
		Write(TEXT(" era=\"before\""));
	Write(TEXT(">"));
	if (date.alternateyear>=0)
		StringCbPrintf(buffer,sizeof(buffer),TEXT("%04i/%02i-%02i-%02i"),date.year,date.alternateyear,date.month,date.day);
	else
		StringCbPrintf(buffer,sizeof(buffer),TEXT("%04i-%02i-%02i"),date.year,date.month,date.day);
	if (time && date.hours>-1)
	{
		if (date.seconds>-1)
			StringCbPrintf(tmp,sizeof(tmp),TEXT("T%02i:%02i:%02i"),date.hours,date.minutes,date.seconds);
		else
			StringCbPrintf(tmp,sizeof(tmp),TEXT("T%02i:%02i"),date.hours,date.minutes);
		StringCbCat(buffer,sizeof(buffer),tmp);
	}
	Write(buffer);
	Write(TEXT("</"));
	Write(tag);
	Write(TEXT(">"));
}

void GENXML::Change(GC_CHANGE &Change)
{
	if (Change.Date.year)
		SimpleDate(Change.Date,TEXT("change"),NULL,TRUE);
}

void GENXML::PersonalName(GC_NAME &name)
{
	static TCHAR szNameStart[]=TEXT("<personalname>");
	static TCHAR szNameEnd[]=TEXT("</personalname>");
	static TCHAR szUnknown[]=TEXT("<np tp=\"unkw\">");
	static TCHAR szGivenName[]=TEXT("<np tp=\"givn\">");
	static TCHAR szSurname[]=TEXT("<np tp=\"surn\">");
	static TCHAR szPrefix[]=TEXT("<np tp=\"pref\">");
	static TCHAR szSuffix[]=TEXT("<np tp=\"sufx\">");
	static TCHAR szArticle[]=TEXT("<np tp=\"art\">");
	static TCHAR szNickname[]=TEXT("<np tp=\"nick\">");
	static TCHAR szPartEnd[]=TEXT("</np>");
	TCHAR tmp[256];
	TCHAR* ptr;
	TCHAR* ptr2;
	STRING conv;

	Write(szNameStart);
	if (ConvData->UseStructuredNames)
	{
		if (name.Npfx!=TEXT(""))
		{
			Write(szPrefix);
			Write(name.Npfx);
			Write(szPartEnd);
		}
		if (name.Givn!=TEXT(""))
		{
			Write(szGivenName);
			Write(name.Givn);
			Write(szPartEnd);
		}
		if (name.Nick!=TEXT(""))
		{
			Write(szNickname);
			Write(name.Nick);
			Write(szPartEnd);
		}
		if (name.Spfx!=TEXT(""))
		{
			Write(szArticle);
			Write(name.Spfx);
			Write(szPartEnd);
		}
		if (name.Surn!=TEXT(""))
		{
			Write(szSurname);
			Write(name.Surn);
			Write(szPartEnd);
		}
		if (name.Nsfx!=TEXT(""))
		{
			Write(szSuffix);
			Write(name.Nsfx);
			Write(szPartEnd);
		}
	}
	else
	{
		name.Name.Get(tmp,sizeof(tmp));
		ptr = StrChr(tmp,'/');
		if (ptr)
			ptr[0] = 0;
		if (lstrlen(tmp))
		{
			if (ConvData->AssumeGivenName)
				Write(szGivenName);
			else
				Write(szUnknown);
			conv = tmp;
			Write(conv);
			Write(szPartEnd);
		}
		if (ptr)
		{
			ptr++;
			ptr2 = StrChr(ptr,'/');
			if (ptr2)
				ptr2[0] = 0;
			if (lstrlen(ptr))
			{
				Write(szSurname);
				conv = ptr;
				Write(conv);
				Write(szPartEnd);
			}
			if (ptr2 && lstrlen(ptr2))
			{
				Write(szUnknown);
				conv = ptr2;
				Write(conv);
				Write(szPartEnd);
			}
		}
	}
	Write(szNameEnd);
}

void GENXML::Place(GC_PLACE &Place)
{
	static TCHAR szPlaceStart[]=TEXT("<place>");
	static TCHAR szPlaceEnd[]=TEXT("</place>");
	static TCHAR szPartStart[]=TEXT("<pnp>");
	static TCHAR szPartEnd[]=TEXT("</pnp>");
	TCHAR tmp[256];
	TCHAR* token;
	STRING conv;	// Necessary to convert text string to valid XML

	if (Place.Place==TEXT(""))
		return;
	Write(szPlaceStart);
	Place.Place.Get(tmp,sizeof(tmp));
	token = wcstok(tmp,TEXT(","));
	do
	{
		while (token[0]==' ')
			token++;
		if (token[0])
		{
			Write(szPartStart);
			conv = token;
			Write(conv);
			Write(szPartEnd);
		}
		token = wcstok(NULL,TEXT(","));
	}
	while (token);
	Write(szPlaceEnd);
}

void GENXML::Address(GC_ADDRESS &Address)
{
	static TCHAR szAddressStart[]=TEXT("<address>");
	static TCHAR szAddressEnd[]=TEXT("</address>");
	static TCHAR szPartStart[]=TEXT("<ap>");
	static TCHAR szPartEnd[]=TEXT("</ap><lf/>");
	static TCHAR szPhoneStart[]=TEXT("<phone>");
	static TCHAR szPhoneEnd[]=TEXT("</phone>");
	static TCHAR szFaxStart[]=TEXT("<fax>");
	static TCHAR szFaxEnd[]=TEXT("</fax>");

	if (!Address.AddressLines.GetSize() && Address.Phon==TEXT("") && Address.Fax==TEXT(""))
		return;
	Write(szAddressStart);
	for (int i=1;i<=Address.AddressLines.GetSize();i++)
	{
		Write(szPartStart);
		Write(*Address.AddressLines.Get(i));
		Write(szPartEnd);
	}
	if (Address.Phon!=TEXT(""))
	{
		Write(szPhoneStart);
		Write(Address.Phon);
		Write(szPhoneEnd);
	}
	if (Address.Fax!=TEXT(""))
	{
		Write(szFaxStart);
		Write(Address.Fax);
		Write(szFaxEnd);
	}
	Write(szAddressEnd);
}

void GENXML::Note(GC_NOTE &Note,VECTOR<GC_NOTEREC*> &Notes, IDTABLE &NoteIDs)
{
	static TCHAR szStart[]=TEXT("<note>");
	static TCHAR szEnd[]=TEXT("</note>");

	if (Note.Note==TEXT(""))
		return;
	Write(szStart);
	NoteText(&Note,Notes,NoteIDs);
	Write(szEnd);
}

void GENXML::NoteText(GC_NOTE* Note,VECTOR<GC_NOTEREC*> &Notes, IDTABLE &NoteIDs)
{
	GC_NOTEREC* nr;
	long i;

	if (Note->NoteID[0])
	{
		i = NoteIDs.Find(Note->NoteID);
		nr = Notes.Get(i);
		Write(nr->Note);
	}
	else
		Write(Note->Note);
}

void GENXML::Excerptref(IDLIST& IDs)
{
	static TCHAR szStart[]=TEXT("<excerptref>");
	static TCHAR szEnd[]=TEXT("</excerptref>");
	TCHAR tmp[32];

	for (int i=1;i<=IDs.GetSize();i++)
	{
		Write(szStart);
		StringCbPrintf(tmp,sizeof(tmp),TEXT("X%i"),IDs.Get(i));
		Write(tmp);
		Write(szEnd);
	}
}

void GENXML::Object(GC_OBJECT &Object,VECTOR<GC_OBJECTREC*> &Objects,IDTABLE &ObjectIDs,VECTOR<GC_NOTEREC*> &Notes,IDTABLE &NoteIDs,int ObjectStatus)
{
	static TCHAR szStart[]=TEXT("<object>");
	static TCHAR szEnd[]=TEXT("</object>");
	static TCHAR szOriginalStart[]=TEXT("<originalfile>");
	static TCHAR szOriginalEnd[]=TEXT("</originalfile>");
	static TCHAR szExternalStart[]=TEXT("<externalfile>");
	static TCHAR szExternalEnd[]=TEXT("</externalfile>");
	static TCHAR szTitleStart[]=TEXT("<title>");
	static TCHAR szTitleEnd[]=TEXT("</title>");
	GC_OBJECTREC* ObjectRecord;
	TCHAR* szFile;
	STRING tmp;
	TCHAR szLocalFile[MAX_PATH];

	Write(szStart);
	if (Object.ObjectID[0])
	{
		long i = ObjectIDs.Find(Object.ObjectID);
		if (i)
		{
			ObjectRecord = Objects.Get(i);
			szFile = StrRChr(ObjectRecord->szPath.Get(),NULL,'\\');
			if (szFile)
				szFile++;
			else
				szFile = ObjectRecord->szPath.Get();
			if (ObjectStatus==1)		// External objects
			{
				GetCurrentDirectory(sizeof(szLocalFile),szLocalFile);
				StringCbCat(szLocalFile,sizeof(szLocalFile),TEXT("\\"));
				StringCbCat(szLocalFile,sizeof(szLocalFile),szFile);
				CopyFile(ObjectRecord->szPath.Get(),szLocalFile,TRUE);
				Write(szExternalStart);
				tmp = szFile;
				Write(tmp);
				Write(szExternalEnd);
			}
			else						// Internal objects
			{
				Write(szOriginalStart);
				tmp = szFile;
				Write(tmp);
				Write(szOriginalEnd);
				ObjectBin(ObjectRecord->szPath.Get());
			}
			Write(szTitleStart);
			Write(ObjectRecord->szTitle);
			Write(szTitleEnd);
			Note(ObjectRecord->Note,Notes,NoteIDs);
		}
	}
	else
	{
		szFile = StrRChr(Object.szPath.Get(),NULL,'\\');
		if (szFile)
			szFile++;
		else
			szFile = Object.szPath.Get();
		if (ObjectStatus==0)			// Internal objects
		{
			Write(szOriginalStart);
			tmp = szFile;
			Write(tmp);
			Write(szOriginalEnd);
			ObjectBin(Object.szPath.Get());
		}
		else							// External objects
		{
			Write(szExternalStart);
			tmp = szFile;
			Write(tmp);
			Write(szExternalEnd);
		}
		Write(szTitleStart);
		Write(Object.szTitle);
		Write(szTitleEnd);
		Note(Object.Note,Notes,NoteIDs);
		/*Write(szExternalStart);
		Write(Object.szPath);
		Write(szExternalEnd);
		Write(szTitleStart);
		Write(Object.szTitle);
		Write(szTitleEnd);
		Note(Object.Note,Notes,NoteIDs);*/
	}
	Write(szEnd);
}

void GENXML::ObjectBin(TCHAR* szPath)
{
	static TCHAR szStart[]=TEXT("<bin>");
	static TCHAR szEnd[]=TEXT("</bin>");
	TCHAR ConvTable[64]={'A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P',
						'Q','R','S','T','U','V','W','X','Y','Z','a','b','c','d','e','f',
						'g','h','i','j','k','l','m','n','o','p','q','r','s','t','u','v',
						'w','x','y','z','0','1','2','3','4','5','6','7','8','9','+','/'};
	TCHAR szText[80];
	unsigned char first,second,third,fourth,tmp;
	HANDLE hFile;
	int size;
	BYTE* data;
	DWORD BytesRead;

	hFile = CreateFile(szPath,GENERIC_READ,0,NULL,OPEN_EXISTING,0,NULL);
	if (hFile==INVALID_HANDLE_VALUE)
		return;
	size = GetFileSize(hFile,NULL);
	data = (BYTE*) VirtualAlloc(NULL,size,MEM_COMMIT,PAGE_READWRITE);
	ReadFile(hFile,data,size,&BytesRead,NULL);
	Write(szStart);
	//unsigned char* input = (unsigned char*) data;
	int i = 0;
	int k = 0;
	while (i<size)
	{
		// Wanted bits: xxxx xx00,0000 0000,0000 0000
		first = data[i];
		first >>= 2;
		szText[k] = ConvTable[first];
		// Wanted bits: 0000 00xx,xxxx 0000,0000 0000
		tmp = data[i];
		tmp <<= 4;
		tmp &= 0x30;//0xc0;
		if (i<size-1)
			second = data[i+1];
		else
			second = 0;
		second >>= 4;
		second += tmp;
		szText[k+1] = ConvTable[second];
		if (i<size-1)
		{
			// Wanted bits: 0000 0000,0000 xxxx,xx00 0000
			tmp = data[i+1];
			tmp <<= 2;
			tmp &= 0x3c;//0xc0;
			if (i<size-2)
				third = data[i+2];
			else
				third = 0;
			third >>= 6;
			third += tmp;
			szText[k+2] = ConvTable[third];
			if (i<size-2)
			{
				// Wanted bits: 0000 0000,0000 0000,00xx xxxx
				fourth = data[i+2];
				fourth &= 0x3f;//0xc0;
				szText[k+3] = ConvTable[fourth];
			}
			else
				szText[k+3] = '=';
		}
		else
		{
			szText[k+2] = '=';
			szText[k+3] = '=';
		}
		k += 4;
		i += 3;
		if (k>=72)
		{
			szText[k] = 0x0d;
			szText[k+1] = 0x0a;
			szText[k+2] = 0;
			Write(szText);
			k = 0;
		}
	}
	if (k)
	{
		szText[k] = 0;
		Write(szText);
	}
	Write(szEnd);
	VirtualFree(data,0,MEM_RELEASE);
	CloseHandle(hFile);
}
